import React from 'react';
import { FormApi, getIn } from 'final-form';
import Button from '@material-ui/core/Button';
import Typography from '@material-ui/core/Typography';
import { withFinalFormField } from '@comparaonline/ui-with-final-form-field';
import { IFieldProps } from '@comparaonline/ui-wizard-step-manager';
import { Card } from '../Card';
import { withStyles } from './styles';

import { AlertSeverity, AlertWarning } from '@comparaonline/ui-form-fields-next';
import { useField } from 'react-final-form';
import { useFormatMessage } from '@comparaonline/react-intl-hooks';
import { PdfModal } from '../PdfModal';

interface CardActionModalProps extends Pick<IFieldProps, 'formApi'> {
  labelDigitalSignature: string;
  labelGenerated: string;
  labelSuccessDigital: string;
  errorText: string;
  getGeneratedValue: (value: string, formApi: FormApi) => void;
  value: string;
  hasError: boolean;
  pathUploadUrl: string;
  classes: Record<string, string>;
  urlPromissoryNote: string;
  title: string;
  subtitle: string;
  buttonEdit: string;
}

const CardActionBase = ({
  classes,
  formApi,
  labelDigitalSignature,
  labelGenerated,
  errorText,
  getGeneratedValue,
  value,
  hasError,
  pathUploadUrl,
  urlPromissoryNote,
  title,
  subtitle,
  buttonEdit
}: CardActionModalProps): JSX.Element => {
  const [isOpenModal, setIsOpenModal] = React.useState<boolean>(true);
  const t = useFormatMessage();
  React.useEffect(() => {
    if (value) setIsOpenModal(() => false);
  }, []);
  const fieldState = formApi.getFieldState(pathUploadUrl);
  const generateValue = getGeneratedValue ? getGeneratedValue(value, formApi) : value;

  const isLoading = fieldState?.data?.loading && generateValue === undefined;
  const showError = hasError && !isLoading;

  const handleCloseModal = () => {
    setIsOpenModal(false);
  };
  const handleOpenModal = () => {
    setIsOpenModal(true);
  };
  const handlePromiseCloseModal = () => {
    handleCloseModal();
    return Promise.resolve();
  };

  return (
    <>
      <Card className={showError ? classes.error : ''}>
        <Typography className={classes.label} variant="body2">
          {labelGenerated}
        </Typography>
        <Button variant="extendedFab" className={classes.buttonOutline} onClick={handleOpenModal}>
          {labelDigitalSignature}
        </Button>
      </Card>
      {showError ? <AlertWarning severity={AlertSeverity.Error} text={t(errorText)} /> : null}
      <PdfModal
        texts={{ title, subtitle, buttonEdit }}
        onSecondaryAction={handlePromiseCloseModal}
        pdfURL={urlPromissoryNote}
        open={isOpenModal}
        onClose={handleCloseModal}
      />
    </>
  );
};

const CardActionModal = withStyles(withFinalFormField(CardActionBase));

export const CardActionFieldModal = ({ texts, ...props }): JSX.Element => {
  const { formApi, pathUploadUrl, validate, pathDownloadUrl } = props;
  const { values } = formApi.getState();
  const value = getIn(values, pathUploadUrl);
  const urlPromissoryNote = getIn(values, pathDownloadUrl);
  const { meta } = useField(pathUploadUrl, {
    validate,
    subscription: { touched: true, error: true, data: true }
  });
  const hasError = Boolean(meta.touched && meta.error?.errorCode);
  const errorText = hasError && meta.error.errorCode;

  return (
    <CardActionModal
      {...props}
      {...texts}
      errorText={errorText}
      hasError={hasError}
      value={value}
      urlPromissoryNote={urlPromissoryNote}
    />
  );
};
