import React from 'react';
import { useField } from 'react-final-form';
import { FormApi, getIn } from 'final-form';
import Button from '@material-ui/core/Button';
import CircularProgress from '@material-ui/core/CircularProgress';
import Typography from '@material-ui/core/Typography';
import { AlertSeverity, AlertWarning } from '@comparaonline/ui-form-fields-next';
import { withFinalFormField } from '@comparaonline/ui-with-final-form-field';
import { IFieldProps } from '@comparaonline/ui-wizard-step-manager';
import { Card } from '../Card';
import { withStyles, WithStyles } from './styles';
import { useFormatMessage } from '@comparaonline/react-intl-hooks';

export interface CardGeneratorProps<Value = string> {
  errorText: string;
  formApi: FormApi;
  hasError: boolean;
  buttonText: string;
  label: string;
  loading: string;
  name: string;
  onClick: IFieldProps['onClick'];
  value: Value;
  getGeneratedValue?: (value: Value, formApi: FormApi) => unknown;
}

const CardGeneratorBase = ({
  classes,
  errorText,
  formApi,
  getGeneratedValue,
  hasError,
  buttonText,
  label,
  loading: loadingText,
  name,
  onClick,
  value
}: WithStyles<CardGeneratorProps>): JSX.Element => {
  const t = useFormatMessage();
  const fieldState = formApi.getFieldState(name);
  const generateValue = getGeneratedValue ? getGeneratedValue(value, formApi) : value;
  const isLoading = fieldState?.data?.loading && generateValue === undefined;
  const showError = hasError && !isLoading;

  const setLoading = (loading: boolean): void => {
    formApi.mutators.setFieldData(name, { loading });
  };

  const handleClick = () => {
    setLoading(true);
    onClick();
  };

  return (
    <>
      <Card className={showError ? classes.error : ''}>
        {!isLoading ? (
          <>
            <Typography
              className={`${classes.msg} ${classes.textColor}`}
              align="center"
              variant="body2"
            >
              {label}
            </Typography>
            <Button variant="extendedFab" className={classes.button} onClick={handleClick}>
              {buttonText}
            </Button>
          </>
        ) : (
          <>
            <CircularProgress className={classes.loading} size="34px" />
            <Typography align="center" variant="body2">
              {loadingText}
            </Typography>
          </>
        )}
      </Card>
      {showError ? <AlertWarning severity={AlertSeverity.Error} text={t(errorText)} /> : null}
    </>
  );
};

const CardGenerator = withStyles(withFinalFormField(CardGeneratorBase));

export const CardGeneratorField = ({ texts, ...props }) => {
  const { formApi, name, validate } = props;
  const { values } = formApi.getState();
  const value = getIn(values, name);
  const { meta } = useField(name, {
    validate,
    subscription: { touched: true, error: true, data: true }
  });

  const hasError = Boolean(meta.touched && meta.error?.errorCode);
  const errorText = hasError && meta.error.errorCode;

  return (
    <CardGenerator {...props} {...texts} errorText={errorText} hasError={hasError} value={value} />
  );
};
