import React, { FC } from 'react';
import { FormApi, getIn } from 'final-form';
import { CicoPaymentMethod } from '@comparaonline/cico-funnel-schema';
import {
  RequiredFileHandlers,
  FileInfoHandler,
  ButtonType
} from '@comparaonline/ui-form-fields-next';
import { useFormatMessage } from '@comparaonline/react-intl-hooks';

import { FieldNames, StatusPromissoryNote } from '../../constants';
import {
  isValidSarlaft,
  FileController,
  isPaymentMethod,
  copyInsuredSarlaftDownloadUrlToUploadUrl
} from '../../helpers';
import { BooleanValues, SarlaftPersonType } from '../../types';
import { DocumentStatus } from './DocumentStatus';
import { CicoMetadata } from './types';

interface DocumentsLoadingProps extends RequiredFileHandlers, FileInfoHandler {
  additionalDocumentFilename: string;
  autoDownload: (formApi: FormApi) => boolean;
  cicoMetadata: CicoMetadata;
  formApi: FormApi;
  showAdditionalDocument: boolean;
  sarlaftSignatureRequired: boolean;
}

const DocumentsLoading: FC<DocumentsLoadingProps> = ({
  additionalDocumentFilename,
  autoDownload,
  cicoMetadata: { offerId, quoteId },
  sarlaftSignatureRequired,
  downloadFile,
  formApi,
  showAdditionalDocument
}) => {
  const { getState: getDocumentsState, setState: setDocumentState } = FileController(
    offerId,
    quoteId
  );
  const documentState = getDocumentsState();
  const t = useFormatMessage();
  const values = formApi.getState().values;

  const hasPayerSarlaft = getIn(values, FieldNames.Insured.IsOwnerThePayer) === BooleanValues.No;
  const payerSarlaftIsValid = hasPayerSarlaft && isValidSarlaft(SarlaftPersonType.Payer, formApi);

  const getUrl = (fieldName: string) => getIn(values, `${fieldName}.downloadUrl`);

  const additionalDocumentFieldName = `${FieldNames.Files.AdditionalDocument}`;
  const additionalDocumentUrl = getIn(values, additionalDocumentFieldName);
  const insuredSarlaftUrl = getUrl(FieldNames.Files.InsuredSarlaft);
  const signedInsuredSarlaftUrl = getIn(values, `${FieldNames.Files.InsuredSarlaft}.uploadUrl`);
  const payerSarlaftUrl = payerSarlaftIsValid && getUrl(FieldNames.Files.PayerSarlaft);
  const promissoryNoteUrl = getUrl(FieldNames.Files.PromissoryNote);
  const promissoryNoteStatus = getIn(values, FieldNames.PromissoryNote.Status);
  const hasPromissoryNoteFailed = promissoryNoteStatus === StatusPromissoryNote.Failed;

  const setDocumentStateAsTrue = (documentURL: string, fieldName: string): void => {
    if (documentURL && !documentState[documentURL]) {
      if (autoDownload(formApi)) {
        downloadFile(t(`files.name.${fieldName}.${ButtonType.Secondary}`), documentURL);
      }
      setDocumentState(documentURL, true);
    }
  };

  if (
    insuredSarlaftUrl !== signedInsuredSarlaftUrl &&
    !sarlaftSignatureRequired &&
    insuredSarlaftUrl
  ) {
    copyInsuredSarlaftDownloadUrlToUploadUrl(formApi);
  }

  setDocumentStateAsTrue(insuredSarlaftUrl, FieldNames.Files.InsuredSarlaft);
  setDocumentStateAsTrue(payerSarlaftUrl, FieldNames.Files.PayerSarlaft);
  setDocumentStateAsTrue(promissoryNoteUrl, FieldNames.Files.PromissoryNote);

  if (showAdditionalDocument) {
    setDocumentStateAsTrue(additionalDocumentUrl, additionalDocumentFieldName);
  }

  return (
    <>
      <DocumentStatus
        isDocumentReady={Boolean(insuredSarlaftUrl)}
        fieldName={FieldNames.Files.InsuredSarlaft}
      />
      {showAdditionalDocument && (
        <DocumentStatus
          isDocumentReady={Boolean(additionalDocumentUrl)}
          fieldName={FieldNames.Files.AdditionalDocument}
          textData={{ filename: additionalDocumentFilename }}
        />
      )}
      {payerSarlaftIsValid && (
        <DocumentStatus
          isDocumentReady={Boolean(payerSarlaftUrl)}
          fieldName={FieldNames.Files.PayerSarlaft}
        />
      )}
      {isPaymentMethod(formApi, CicoPaymentMethod.Financed) && (
        <DocumentStatus
          isDocumentReady={Boolean(promissoryNoteUrl)}
          fieldName={FieldNames.Files.PromissoryNote}
          hasDocumentFailed={hasPromissoryNoteFailed}
        />
      )}
    </>
  );
};

export { DocumentsLoading };
