import React, { useState } from 'react';
import { withApollo, WithApolloClient } from 'react-apollo';
import { FormProps, AnyObject } from 'react-final-form';
import { useFormatMessage } from '@comparaonline/react-intl-hooks';
import { FieldSetNext as Fieldset } from '@comparaonline/ui-fieldset';
import { Checkout, CicoSarlaftType } from '@comparaonline/cico-funnel-schema';
import { CicoForm } from '@comparaonline/quote-forms-schema/dist/schema';
import { WizardForm } from '@comparaonline/ui-wizard-form-next';
import {
  StepManager,
  SectionManager,
  LineSeparator,
  DotStepper,
  IStepChangeListenerProps
} from '@comparaonline/ui-wizard-step-manager';
import { RequiredFileHandlers } from '@comparaonline/ui-form-fields-next';

import { FieldResolver } from './FieldResolver';
import { LoadingBar } from './LoadingBar';
import {
  handleFormLoad,
  onValuesUpdate,
  parseRequestedForm,
  prepareFirestoreData,
  prepareFirestoreDataHandler,
  decorateFileHandlers,
  saveStepIdInSarlafts
} from '../helpers';
import { stepsConfig } from '../steps';
import { getFieldSubscriptions, onFormValidSubscription } from '../subscriptions';
import {
  AllowedPaymentMethod,
  CreateInspection,
  Offer,
  Question,
  Submitter,
  StepChangeHandler
} from '../types';
import { createRedirectToQuote } from '../mutators';
import { StepIds } from '../constants';

interface FileHandler {
  fileHandler: RequiredFileHandlers;
}

interface WizardFormProps extends FileHandler {
  bffURL: string;
  fulfillmentPresenterURL: string;
  createInspection: Submitter;
  showInspectionCenter(): void;
  extended: boolean;
  handleSubmit: Submitter;
  inspectionCentersUrl: string;
  isDigitalSignatureEnabled: boolean;
  sarlaftSignatureRequired?: boolean;
  offer: Offer;
  ownerViewing: boolean;
  paymentMethods: AllowedPaymentMethod[];
  quoteForm: CicoForm;
  additionalDocumentFilename?: string;
  formProps?: Pick<FormProps<AnyObject>, Exclude<keyof FormProps<AnyObject>, 'onSubmit'>>;
  initialStep?: StepIds;
  onFormValid?: Submitter[];
  onStepChange?: StepChangeHandler;
  requireCarInformation?: string[];
  showAdditionalDocument?: boolean;
  values?: Partial<Checkout>;
  optionalSarlaftFields?: string[];
}

const stepChangeHandler = (
  onStepChange: StepChangeHandler | undefined,
  saveCheckout: Submitter
) => (props: IStepChangeListenerProps) => {
  const currentStepId = props.stepManagerProps.steps[0].id;
  const { formApi } = props;

  saveStepIdInSarlafts(currentStepId, formApi);
  const { values } = formApi.getState();
  const { fireStoreData, collections } = prepareFirestoreData(values, formApi);
  saveCheckout(fireStoreData, collections);

  return (
    onStepChange &&
    onStepChange({
      formProps: props,
      transformValues: formValues => prepareFirestoreData(formValues, formApi)
    })
  );
};

const WizardFormBase: React.FC<WithApolloClient<WizardFormProps>> = ({
  bffURL,
  fulfillmentPresenterURL,
  createInspection,
  client,
  extended,
  fileHandler,
  handleSubmit,
  initialStep,
  isDigitalSignatureEnabled,
  sarlaftSignatureRequired,
  offer,
  onStepChange,
  ownerViewing,
  paymentMethods,
  quoteForm,
  showInspectionCenter,
  additionalDocumentFilename = '',
  formProps = {},
  onFormValid = [],
  requireCarInformation = [],
  showAdditionalDocument = false,
  values = {},
  optionalSarlaftFields
}) => {
  const translate = useFormatMessage();
  const decoratedFileHandlers = decorateFileHandlers(fileHandler, translate);
  const [questions, setQuestions] = useState<Question[] | undefined>();
  const { mutators = {} } = formProps;
  const onCreateInspection: CreateInspection = (formValues, formApi) => {
    const { fireStoreData, collections } = prepareFirestoreData(formValues, formApi);
    return createInspection(fireStoreData, collections);
  };

  const getSarlaftType = (formValues: Partial<Checkout>): CicoSarlaftType =>
    formValues.sarlaft?.insured.type || CicoSarlaftType.LONG;

  const steps = stepsConfig({
    additionalDocumentFilename,
    bffURL,
    fulfillmentPresenterURL,
    offer,
    isDigitalSignatureEnabled,
    sarlaftSignatureRequired,
    ownerViewing,
    paymentMethods,
    questions,
    quoteForm,
    requireCarInformation,
    showAdditionalDocument,
    showInspectionCenter,
    initialStep,
    createInspection: onCreateInspection,
    fileHandler: decoratedFileHandlers,
    onGetQuestions: setQuestions,
    sarlaftType: getSarlaftType(values),
    optionalSarlaftFields
  });

  const initialStepIndex = steps.findIndex(step => step.id === initialStep);
  const onStepChangeHandler = stepChangeHandler(onStepChange, onFormValid[0]);

  return (
    <WizardForm
      fieldSubscriptions={getFieldSubscriptions()}
      formProps={{
        ...formProps,
        onSubmit: prepareFirestoreDataHandler(handleSubmit),
        initialValues: parseRequestedForm(values, offer),
        mutators: {
          ...mutators,
          redirectToQuote: createRedirectToQuote(quoteForm.id)
        }
      }}
      formSubscriptions={onFormValidSubscription(onFormValid)}
      onFormLoad={handleFormLoad(client, bffURL)}
      onValuesUpdate={onValuesUpdate}
    >
      <LoadingBar />
      <StepManager
        onStepChange={onStepChangeHandler}
        steps={steps}
        dependencies={[questions]}
        extended={extended}
        initialStep={initialStep}
        initialStepIndex={initialStepIndex === -1 ? 0 : initialStepIndex}
      >
        <Fieldset title={translate('extendedTitle')} extended={!extended}>
          <SectionManager
            fieldResolver={FieldResolver}
            connector={<LineSeparator />}
            customStepper={DotStepper}
          />
        </Fieldset>
      </StepManager>
    </WizardForm>
  );
};

const WithApolloWizardFormBase = withApollo<WizardFormProps>(WizardFormBase);

export { WithApolloWizardFormBase as WizardFormBase, WizardFormProps };
