import React, { useState } from 'react';

import { faTimes } from '@fortawesome/pro-regular-svg-icons/faTimes';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import Button from '@material-ui/core/Button';
import Dialog, { DialogProps } from '@material-ui/core/Dialog';
import IconButton from '@material-ui/core/IconButton';
import MobileStepper from '@material-ui/core/MobileStepper';
import Typography from '@material-ui/core/Typography';
import { withTheme, WithTheme } from '@material-ui/core/styles';
import withMobileDialog, { InjectedProps } from '@material-ui/core/withMobileDialog';

import { useFormatMessage } from '@comparaonline/react-intl-hooks';
import { CloudinaryImage } from '@comparaonline/cloudinary-image';
import { Styles, withStyles } from './styles';

interface WizardModalProps {
  onClose: () => void;
  open: DialogProps['open'];
  steps: {
    image: string;
    text: string;
  }[];
}

type Props = WizardModalProps & Styles & WithTheme;

const FIRST_STEP = 0;
const ONE_STEP = 1;

const WizardModalBase: React.FC<Props & InjectedProps> = ({
  classes,
  fullScreen,
  onClose,
  open,
  steps
}) => {
  const t = useFormatMessage();
  const [currentStep, setCurrentStep] = useState<number>(FIRST_STEP);
  const maxSteps = steps.length;

  const handleBack = () => setCurrentStep(currentStep - ONE_STEP);
  const handleNext = () => {
    const isLastStep = currentStep === maxSteps - ONE_STEP;
    isLastStep ? handleClose() : setCurrentStep(currentStep + ONE_STEP);
  };
  const handleClose = () => {
    onClose();
    setCurrentStep(FIRST_STEP);
  };

  return (
    <Dialog
      classes={{
        paper: classes.paper
      }}
      fullScreen={fullScreen}
      open={open}
      onClose={handleClose}
    >
      <IconButton onClick={handleClose} className={classes.closeButton}>
        <FontAwesomeIcon icon={faTimes} className={classes.closeIcon} />
      </IconButton>
      <CloudinaryImage name={steps[currentStep].image} className={classes.image} loading="lazy" />
      <Typography
        variant="body1"
        className={classes.text}
        dangerouslySetInnerHTML={{ __html: t(steps[currentStep].text) }}
      />
      <MobileStepper
        steps={maxSteps}
        position="static"
        activeStep={currentStep}
        classes={{
          dot: classes.dot,
          dotActive: classes.dotActive,
          dots: classes.dots,
          root: classes.stepper
        }}
        nextButton={
          <Button
            variant="extendedFab"
            className={classes.next}
            color="primary"
            onClick={handleNext}
          >
            {t('WizardModal.next')}
          </Button>
        }
        backButton={
          <Button
            variant="extendedFab"
            className={classes.back}
            onClick={handleBack}
            disabled={currentStep === FIRST_STEP}
          >
            {t('WizardModal.back')}
          </Button>
        }
      />
    </Dialog>
  );
};

const WizardModal = withTheme()(withStyles(withMobileDialog<Props>()(WizardModalBase)));

export { WizardModal };
