import { FormApi, getIn } from 'final-form';
import {
  QUESTION_FIELDNAME_PREFIX,
  MAX_ATTEMPTS_TO_VALIDATE_CODE,
  MAX_RESEND_VERIFICATION_CODE,
  SECOND_AS_MILISECOND,
  SARFLAFT_FILENAME,
  PROMISSORY_NOTE_FILENAME,
  FieldNames,
  StatusDigitalSignature
} from '../constants';
import { IdentificationType } from '@comparaonline/common-schema';
import { isPaymentMethod } from './payment';
import { CicoPaymentMethod } from '@comparaonline/cico-funnel-schema';
import { hasFailedPromissoryNote, hasPromissoryNote } from './promissoryNote';

const VALID_STATUS_FOR_INITIALIZATION = [
  StatusDigitalSignature.Successful,
  StatusDigitalSignature.WaitingCodeVerification,
  StatusDigitalSignature.WaitingChallengeQuestionsVerification
];
const INVALID_STATUS_DIGITAL_SIGNATURE = [
  StatusDigitalSignature.Expired,
  StatusDigitalSignature.Failed
];

const VALID_IDENTIFICATION_TYPE = [
  IdentificationType.NationalId,
  IdentificationType.ForeignNationalId
];

type DigitalSignatureHelper<Response = boolean> = (formApi: FormApi) => Response;
type AnswerForRequest = { questionId: string; selectedAnswerId: string };

export const canInitializeIdentityValidation = (formApi: FormApi, isEnabled: boolean): boolean => {
  if (!isEnabled) return false;

  const { values } = formApi.getState();
  const identificationType = getIn(values, FieldNames.Insured.Person.IdentificationType);
  const status = getStatus(formApi);
  const noHasStatus = !Boolean(status);
  const failedPromissoryNote = hasFailedPromissoryNote(formApi);

  const hasValidIdentificationType = VALID_IDENTIFICATION_TYPE.includes(identificationType);
  const hasValidStatus = noHasStatus || VALID_STATUS_FOR_INITIALIZATION.includes(status);

  return hasValidIdentificationType && hasValidStatus && !failedPromissoryNote;
};

export const canGoToChallengeQuestionFlow: DigitalSignatureHelper = formApi =>
  Boolean(getRemainingAttemptsToValidate(formApi)) &&
  StatusDigitalSignature.Successful !== getStatus(formApi);

export const getAnswersForRequest: DigitalSignatureHelper<AnswerForRequest[]> = formApi => {
  const { values } = formApi.getState();
  const answersValue = getIn(values, FieldNames.IdentityValidation.IdentityValidationAnswers);
  const answers = Object.keys(answersValue).map(key => ({
    questionId: key.replace(QUESTION_FIELDNAME_PREFIX, ''),
    selectedAnswerId: answersValue[key]
  }));
  return answers;
};

export const getRemainingAttemptsToValidate: DigitalSignatureHelper<number> = formApi => {
  const { values } = formApi.getState();
  const retriedCodeValidations =
    getIn(values, FieldNames.IdentityValidation.RetriedCodeValidations) || 0;
  const attemptsRemaining = Math.max(MAX_ATTEMPTS_TO_VALIDATE_CODE - retriedCodeValidations, 0);
  return attemptsRemaining;
};

export const hasRemainingAttemptsToValidate: DigitalSignatureHelper = formApi =>
  getRemainingAttemptsToValidate(formApi) > 0;

export const canResendCode: DigitalSignatureHelper = formApi => {
  const resendedCodeNumber = getResendedCode(formApi);
  return resendedCodeNumber < MAX_RESEND_VERIFICATION_CODE;
};

export const isLastResendCode: DigitalSignatureHelper = formApi => {
  const resendedCodeNumber = getResendedCode(formApi);
  const resendSlotAvailable = MAX_RESEND_VERIFICATION_CODE - resendedCodeNumber;
  return resendSlotAvailable === 1;
};

export const getDownloadUrl = (fieldName: string, values) =>
  getIn(values, `${fieldName}.downloadUrl`);

export const getFilesInformation = (
  formApi: FormApi,
  sarlaftSignatureRequired: boolean,
  additionalDocumentName?: string
) => {
  const { values } = formApi.getState();
  const sarlaftUrl = getDownloadUrl(FieldNames.Files.InsuredSarlaft, values);
  const isFinanced = isPaymentMethod(formApi, CicoPaymentMethod.Financed);

  const docsData: { docUrl: string; fileName: string }[] = [];

  if (sarlaftSignatureRequired) {
    docsData.push({
      docUrl: sarlaftUrl,
      fileName: SARFLAFT_FILENAME
    });
  }

  if (isFinanced) {
    const promissoryNoteUrl = getDownloadUrl(FieldNames.Files.PromissoryNote, values);
    docsData.push({
      docUrl: promissoryNoteUrl,
      fileName: PROMISSORY_NOTE_FILENAME
    });
  }

  if (additionalDocumentName) {
    docsData.push({
      docUrl: getIn(values, `${FieldNames.Files.InsuredSarlaft}.additionalDocumentUrl`),
      fileName: normalizeAdditionalDocumentFilename(additionalDocumentName)
    });
  }

  return docsData;
};

export const getIdIdentityValidation: DigitalSignatureHelper<string> = formApi => {
  const { values } = formApi.getState();
  return getIn(values, FieldNames.IdentityValidation.Id);
};

export const getResendedCode: DigitalSignatureHelper<number> = formApi => {
  const { values } = formApi.getState();
  const resendedCode = getIn(values, FieldNames.IdentityValidation.ResendCode);
  return Number(resendedCode || 0);
};

export const getStatus: DigitalSignatureHelper<StatusDigitalSignature> = formApi => {
  const { values } = formApi.getState();
  return getIn(values, FieldNames.IdentityValidation.Status);
};

export const isExpiredIdentityValidation: DigitalSignatureHelper = formApi =>
  getStatus(formApi) === StatusDigitalSignature.Expired;

export const isFailureIdentityValidation: DigitalSignatureHelper = formApi =>
  getStatus(formApi) === StatusDigitalSignature.Failed;

export const isInvalidIdentityValidation: DigitalSignatureHelper = formApi =>
  INVALID_STATUS_DIGITAL_SIGNATURE.includes(getStatus(formApi));

export const isValidIdentityValidation: DigitalSignatureHelper = formApi =>
  getStatus(formApi) === StatusDigitalSignature.Successful;

export const isTimeExpired: DigitalSignatureHelper = formApi => {
  const { values } = formApi.getState();
  const expirationDate = getIn(values, FieldNames.IdentityValidation.ExpirationDate);
  const expirationTime = new Date(expirationDate).getTime();
  const currentDate = new Date();
  const currentTime = currentDate.getTime();
  const isExpired =
    currentTime > expirationTime || expirationTime - currentTime < SECOND_AS_MILISECOND;
  return isExpired;
};

export const canEnableQuestionsFlowSubmitButton = (formApi: FormApi) => {
  return isTimeExpired(formApi) || getStatus(formApi) === StatusDigitalSignature.Failed;
};

export const canRenderDigitalSignatureField = (
  formApi: FormApi,
  isDigitalSignatureEnabled: boolean,
  documentUrlPath: string
): boolean => {
  const { values } = formApi.getState();
  const documentUrl = getIn(values, documentUrlPath);
  return (
    canInitializeIdentityValidation(formApi, isDigitalSignatureEnabled) && Boolean(documentUrl)
  );
};

export const canRenderSignButton = (
  formApi: FormApi,
  isEnabled: boolean,
  requireAdditionalDocument: boolean
): boolean => {
  const { values } = formApi.getState();

  const additionalDocumentFileExist = getIn(
    values,
    `${FieldNames.Files.InsuredSarlaft}.additionalDocumentUrl`
  );
  const promissoryNoteFileExist = hasPromissoryNote(formApi);
  const requirePromissoryNoteFile = isPaymentMethod(formApi, CicoPaymentMethod.Financed);

  const isDigitalSignatureEnable = canInitializeIdentityValidation(formApi, isEnabled);
  const sarlaftFileExist = Boolean(getDownloadUrl(FieldNames.Files.InsuredSarlaft, values));
  const commonConditionAreOk = isDigitalSignatureEnable && sarlaftFileExist;
  const filesAreNotSigned = !areFilesSigned(
    values,
    requirePromissoryNoteFile,
    requireAdditionalDocument
  );

  return (
    filesAreNotSigned &&
    commonConditionAreOk &&
    (requirePromissoryNoteFile ? promissoryNoteFileExist : true) &&
    (requireAdditionalDocument ? additionalDocumentFileExist : true)
  );
};

const areFilesSigned = (values, requirePromissoryNote, requireAdditionalDocument) => {
  const additionalDocumentSigned = getIn(
    values,
    `${FieldNames.Files.InsuredSarlaft}.additionalSignedDocumentUrl`
  );
  const insuredSarlaftSigned = getIn(values, `${FieldNames.Files.InsuredSarlaft}.uploadUrl`);
  const promissoryNoteSigned = getIn(values, `${FieldNames.Files.PromissoryNote}.uploadUrl`);

  const additionalDocumentIsOk = requireAdditionalDocument ? additionalDocumentSigned : true;
  const promissoryNoteIsOk = requirePromissoryNote ? promissoryNoteSigned : true;

  return insuredSarlaftSigned && additionalDocumentIsOk && promissoryNoteIsOk;
};

export const normalizeAdditionalDocumentFilename = (filename: string) => {
  const extension = '.pdf';
  return filename.endsWith(extension) ? filename : `${filename}${extension}`;
};

export const insuredSarlaftGenerated = (formApi: FormApi) => {
  const { values } = formApi.getState();
  return !!getIn(values, `${FieldNames.Files.InsuredSarlaft}.downloadUrl`);
};

export const signedInsuredSarlaftUploaded = (formApi: FormApi) => {
  const { values } = formApi.getState();
  return !!getIn(values, `${FieldNames.Files.InsuredSarlaft}.uploadUrl`);
};

export const copyInsuredSarlaftDownloadUrlToUploadUrl = (formApi: FormApi) => {
  const { values } = formApi.getState();
  const downloadUrl = getIn(values, `${FieldNames.Files.InsuredSarlaft}.downloadUrl`);
  formApi.change(`${FieldNames.Files.InsuredSarlaft}.uploadUrl`, downloadUrl);
  formApi.change(`${FieldNames.Files.InsuredSarlaft}.uploadDate`, new Date());
  formApi.change(`${FieldNames.Files.InsuredSarlaft}.fileName`, SARFLAFT_FILENAME);
};
