import { FormApi, getIn } from 'final-form';
import { FieldNames, StatusDigitalSignature, StepIds } from '../../constants';
import { fetchBFF } from './bffapi';
import {
  getAnswersForRequest,
  getFilesInformation,
  normalizeAdditionalDocumentFilename
} from '../digitalSignature';
import { SARFLAFT_FILENAME, PROMISSORY_NOTE_FILENAME } from '../../constants/digitalSignature';
import {
  BypassCodeValidation,
  BypassCodeValidationResponse,
  DigitalSignatureServiceMethod,
  InitIdentityValidationCodeResponse,
  SendCodeResponse,
  VerifyAndSignMethod,
  VerifyAndSignResponse,
  SignedDocument
} from './types';

export const bypassCodeValidation: BypassCodeValidation = async (formApi, bffURL) => {
  const identityValidationId = getFieldValue(formApi, FieldNames.IdentityValidation.Id);
  const URL = getEndpointURL(bffURL, 'bypass-code-verification');

  try {
    const response = await fetchBFF<BypassCodeValidationResponse>(URL, { identityValidationId });

    formApi.batch(() => {
      formApi.change(FieldNames.FieldsNoSave.StatusInputCode, '');
      formApi.change(FieldNames.IdentityValidation.ExpirationDate, response.expirationDate);
      formApi.change(FieldNames.IdentityValidation.Status, response.status);
    });

    return {
      ok: true,
      questions: response.questions
    };
  } catch (error) {
    formApi.change(FieldNames.IdentityValidation.Status, error.status);
    return {
      ok: false
    };
  }
};

export const initIdentityValidation = async (formApi, bffURL, formId) => {
  const identificationNumber = getFieldValue(
    formApi,
    FieldNames.Insured.Person.IdentificationNumber
  );
  const identificationType = getFieldValue(formApi, FieldNames.Insured.Person.IdentificationType);
  const lastName = getFieldValue(formApi, FieldNames.Insured.Person.LastName);
  const identificationExpeditionDate = getFieldValue(formApi, FieldNames.Insured.Person.DateIssued);
  const phoneNumber = getFieldValue(formApi, FieldNames.Insured.Person.Phone);
  const URL = getEndpointURL(bffURL, 'start-identity-validation');

  try {
    const response = (await fetchBFF(URL, {
      identificationExpeditionDate,
      identificationNumber,
      identificationType,
      lastName,
      phoneNumber,
      formId
    })) as InitIdentityValidationCodeResponse;

    formApi.batch(() => {
      formApi.change(FieldNames.IdentityValidation.Id, response.identityValidationId);
      formApi.change(FieldNames.IdentityValidation.ExpirationDate, response.expirationDate);
      formApi.change(FieldNames.IdentityValidation.Status, response.status);
    });

    const questions = (response as BypassCodeValidationResponse).questions || undefined;
    const hasQuestions = Boolean(questions);

    return {
      questions,
      ok: true,
      step: hasQuestions
        ? StepIds.IdentityValidationChallengeQuestions
        : StepIds.IdentityValidationCode
    };
  } catch {
    formApi.change(FieldNames.IdentityValidation.Status, StatusDigitalSignature.Failed);
    return { ok: false };
  }
};

export const reSendCode: DigitalSignatureServiceMethod<boolean> = async (formApi, bffURL) => {
  const identityValidationId = getFieldValue(formApi, FieldNames.IdentityValidation.Id);
  const phoneNumber = getFieldValue(formApi, FieldNames.Insured.Person.Phone);
  const URL = getEndpointURL(bffURL, 'send-verification-code');

  try {
    const response = await fetchBFF<SendCodeResponse>(URL, {
      identityValidationId,
      phoneNumber
    });

    formApi.batch(() => {
      increaseCounter(formApi, FieldNames.IdentityValidation.ResendCode);
      formApi.change(FieldNames.IdentityValidation.ExpirationDate, response.expirationDate);
      formApi.change(FieldNames.IdentityValidation.Status, response.status);
    });

    return true;
  } catch (error) {
    formApi.change(FieldNames.IdentityValidation.Status, error.status);
    return false;
  }
};

export const verifyAnswersAndSign: VerifyAndSignMethod = async (
  formApi,
  bffURL,
  sarlaftSignatureRequired,
  additionalDocumentName = ''
) => {
  const files = getFilesInformation(formApi, sarlaftSignatureRequired, additionalDocumentName);
  const answers = getAnswersForRequest(formApi);
  const identityValidationId = getFieldValue(formApi, FieldNames.IdentityValidation.Id);
  const URL = getEndpointURL(bffURL, 'validate-challenge-questions-answers-and-sign-documents');

  try {
    const response = await fetchBFF<VerifyAndSignResponse>(URL, {
      answers,
      identityValidationId,
      files
    });

    formApi.batch(() => {
      updateValuesOnVerification(formApi, response, additionalDocumentName);
    });

    return true;
  } catch (_error) {
    formApi.change(FieldNames.IdentityValidation.Status, StatusDigitalSignature.Failed);
    return false;
  }
};

export const verifyCodeAndSign: VerifyAndSignMethod = async (
  formApi,
  bffURL,
  sarlaftSignatureRequired,
  additionalDocumentName = ''
) => {
  const verificationCode = getFieldValue(formApi, FieldNames.IdentityValidation.Code);
  const files = getFilesInformation(formApi, sarlaftSignatureRequired, additionalDocumentName);
  const identityValidationId = getFieldValue(formApi, FieldNames.IdentityValidation.Id);
  const URL = getEndpointURL(bffURL, 'validate-verification-code-and-sign-documents');

  try {
    const response = await fetchBFF<VerifyAndSignResponse>(URL, {
      identityValidationId,
      verificationCode,
      files
    });

    formApi.batch(() => {
      updateValuesOnVerification(formApi, response, additionalDocumentName);
    });

    return true;
  } catch (error) {
    formApi.batch(() => {
      increaseCounter(formApi, FieldNames.IdentityValidation.RetriedCodeValidations);
      formApi.change(FieldNames.IdentityValidation.Status, StatusDigitalSignature.Failed);
    });

    return false;
  }
};

function increaseCounter(formApi: FormApi, fieldName: string) {
  const value = Number(getFieldValue(formApi, fieldName) || 0);
  formApi.change(fieldName, value + 1);
}

function getEndpointURL(BFFURL: string, endpoint: string) {
  return `${BFFURL}/digital-signature/${endpoint}`;
}

function getFieldValue(formApi: FormApi, fieldName: string) {
  const { values } = formApi.getState();
  return getIn(values, fieldName);
}

function getSignedDocByFilename(signedDocs: SignedDocument[], filename: string): SignedDocument {
  return signedDocs.find(doc => doc.fileName === filename) as SignedDocument;
}

function cleanTemporalDigitalSignatureData(formApi: FormApi) {
  formApi.change(FieldNames.IdentityValidation.Code, null);
}

function saveSignedDocAsFormValue(
  filename: string,
  { downloadUrl }: SignedDocument,
  formApi: FormApi
) {
  formApi.change(`${filename}.uploadDate`, new Date());
  formApi.change(`${filename}.uploadUrl`, downloadUrl);
  formApi.change(`${filename}.digitallySigned`, true);
}

function updateValuesOnVerification(
  formApi: FormApi,
  response: VerifyAndSignResponse,
  additionalDocumentName: string
) {
  const status = response.status || StatusDigitalSignature.Successful;
  const sarlaftSigned = getSignedDocByFilename(response.signedDocuments, SARFLAFT_FILENAME);
  const promissoryNoteSigned = getSignedDocByFilename(
    response.signedDocuments,
    PROMISSORY_NOTE_FILENAME
  );
  const additionalSignedDocument = getSignedDocByFilename(
    response.signedDocuments,
    normalizeAdditionalDocumentFilename(additionalDocumentName)
  );

  formApi.change(FieldNames.IdentityValidation.Status, status);
  cleanTemporalDigitalSignatureData(formApi);

  if (sarlaftSigned) {
    saveSignedDocAsFormValue(FieldNames.Files.InsuredSarlaft, sarlaftSigned, formApi);
  }

  if (promissoryNoteSigned) {
    saveSignedDocAsFormValue(FieldNames.Files.PromissoryNote, promissoryNoteSigned, formApi);
  }

  if (additionalSignedDocument) {
    formApi.change(
      `${FieldNames.Files.InsuredSarlaft}.additionalSignedDocumentUrl`,
      additionalSignedDocument.downloadUrl
    );
  }
}
