import { FormApi, getIn } from 'final-form';
import { FieldNames } from '../constants';
import { IdentificationType } from '@comparaonline/common-schema';
import { IdentificationDocumentVersion } from '../types';

type ErrorItem = { message: string };
type FailedValidationResult = {
  type: string;
  origin: string;
  level: string;
  name: string;
  message: string;
  errorMessages: (string | ErrorItem)[];
  stack: string;
  statusCode: number;
};

const formatFailedValidationResult = (validationResult: FailedValidationResult): string[] => {
  if (!validationResult.errorMessages || !validationResult.errorMessages.length)
    return [validationResult.message];
  return validationResult.errorMessages.map(errorMessage =>
    typeof errorMessage === 'string' ? errorMessage : errorMessage.message
  );
};
export const validateDocuments = async (
  formApi: FormApi,
  formId: string,
  fulfillmentPresenterURL: string
) => {
  formApi.batch(() => {
    formApi.change(FieldNames.DocumentsValidationResult.Result, '');
    formApi.change(FieldNames.DocumentsValidationResult.Description, []);
  });
  const body = JSON.stringify({
    formId
  });
  const response = await fetch(
    `${fulfillmentPresenterURL}/car-insurance/co/checkout/documents/validation`,
    {
      body,
      method: 'POST',
      headers: {
        'Content-Type': 'application/json'
      }
    }
  );

  if (response.status === 200) {
    formApi.batch(() => {
      formApi.change(FieldNames.DocumentsValidationResult.Result, 'success');
      formApi.change(
        FieldNames.DocumentsValidationResult.Description,
        'Los documentos fueron validados exitosamente'
      );
    });
    return;
  }

  if (response.status === 400) {
    const failedValidationResult: FailedValidationResult = await response.json();
    const formattedFailedValidationResult = formatFailedValidationResult(failedValidationResult);
    formApi.batch(() => {
      formApi.change(FieldNames.DocumentsValidationResult.Result, 'error');
      formApi.change(
        FieldNames.DocumentsValidationResult.Description,
        formattedFailedValidationResult
      );
    });
    return;
  }

  formApi.batch(() => {
    formApi.change(FieldNames.DocumentsValidationResult.Result, 'error');
    formApi.change(
      FieldNames.DocumentsValidationResult.Description,
      'Ocurrió un error desconocido al validar documentos'
    );
  });
};

export const canValidateDocuments = (formApi: FormApi) => {
  const { values } = formApi.getState();
  const identificationType = getIn(values, FieldNames.Insured.Person.IdentificationType);
  return identificationType === IdentificationType.NationalId;
};

export const shouldDisplayIdentificationDocumentVersionSelector = (formApi: FormApi) => {
  const { values } = formApi.getState();
  const identificationType = getIn(values, FieldNames.Insured.Person.IdentificationType);
  return identificationType === IdentificationType.NationalId && canValidateDocuments(formApi);
};

export const shouldDisplayBackOfIdentificationDocumentInput = (formApi: FormApi) => {
  const { values } = formApi.getState();
  const identificationDocumentVersion = getIn(values, FieldNames.IdentificationDocumentVersion);

  if (identificationDocumentVersion === IdentificationDocumentVersion.New) {
    return false;
  }

  return canValidateDocuments(formApi);
};
