import { CicoSarlaftType } from '@comparaonline/cico-funnel-schema';
import { Gender, IdentificationType } from '@comparaonline/common-schema';
import { DEFAULT_FINANCIAL_PRODUCT_CURRENCY } from '../constants';
import { CardButtonOptionItem, Offer, AllowedPaymentMethod } from '../types';
import { getInspectionTypeDefaultValue } from './getInspectionType';
import { getPaymentMethodOptions } from './payment';

const parseDateString = (dateISO: string): string => {
  if (dateISO.lastIndexOf('T') === -1) {
    return dateISO;
  }

  return dateISO
    .slice(0, dateISO.lastIndexOf('T'))
    .split('-')
    .reverse()
    .join('/');
};

const SHORT_SARLAFT_VALUE = 'SHORT';
const isShortSarlaft = (sarlaftCode: string): boolean =>
  sarlaftCode?.toLocaleUpperCase() === SHORT_SARLAFT_VALUE;

const SARLAFT_CODE = 'sarlaft-type-co';
const getSarlaftType = ({ offer, quote, sarlaftConfig }): CicoSarlaftType => {
  const { insuranceVehicleLimit, yearlyFeeLimit } = sarlaftConfig;
  const {
    product: {
      mainAttribute: {
        value: { total: totalPrice }
      },
      productAttributes
    }
  } = offer;

  const {
    car: { valuation }
  } = quote;

  const sarlaftTypeAttribute = productAttributes?.find(
    (attr: any) => attr?.entity?.code === SARLAFT_CODE
  );

  if (!isShortSarlaft(sarlaftTypeAttribute?.value)) return CicoSarlaftType.LONG;
  if (Number(totalPrice) > yearlyFeeLimit || Number(valuation) > insuranceVehicleLimit)
    return CicoSarlaftType.LONG;

  return CicoSarlaftType.SHORT;
};

interface SarlaftConfig {
  insuranceVehicleLimit: number;
  yearlyFeeLimit: number;
}
export const getInitialCheckout = (
  quoteForm,
  paymentMethods: AllowedPaymentMethod[],
  offer: Offer,
  sarlaftConfig: SarlaftConfig
) => {
  const {
    birthdate,
    car,
    email,
    firstName,
    lastName,
    phone,
    typeOfIdentification,
    numberOfIdentification: identificationNumber
  } = quoteForm;

  const allowedPaymentMethods: CardButtonOptionItem[] = getPaymentMethodOptions(
    paymentMethods,
    offer
  );

  const gender: Gender = Object.values(Gender)[quoteForm.gender];
  const identificationType: IdentificationType = Object.values(IdentificationType)[
    typeOfIdentification
  ];

  const sarlaftType = getSarlaftType({ offer, quote: quoteForm, sarlaftConfig });
  return {
    inspection: {
      name: firstName,
      phone,
      type: getInspectionTypeDefaultValue(offer.product),
      inspectionId: '',
      orderCreatedWithType: ''
    },
    sarlaft: {
      insured: {
        type: sarlaftType,
        foreignTaxation: {
          financialProductCurrency: DEFAULT_FINANCIAL_PRODUCT_CURRENCY,
          hasFinancialProducts: 0,
          hasForeignCurrencyOperations: 0
        },
        person: {
          birthDate: parseDateString(birthdate),
          email,
          firstName,
          gender,
          identificationNumber,
          lastName,
          phone,
          identificationType
        },
        publicResources: {
          hasPublicLink: 0,
          hasPublicPosition: 0,
          isPubliclyRecognized: 0,
          managePublicResources: 0
        },
        work: {
          assetValue: String(car.valuation)
        }
      },
      payer: {
        type: sarlaftType,
        foreignTaxation: {
          hasFinancialProducts: 0,
          hasForeignCurrencyOperations: 0
        },
        publicResources: {
          hasPublicLink: 0,
          hasPublicPosition: 0,
          isPubliclyRecognized: 0,
          managePublicResources: 0
        }
      }
    },
    payment: {
      method: allowedPaymentMethods[0] ? allowedPaymentMethods[0].value : undefined
    }
  };
};
