import React from 'react';
import { AnyObject, FormApi, getIn } from 'final-form';
import { CicoPaymentMethod } from '@comparaonline/cico-funnel-schema';
import { CloudinaryImage } from '@comparaonline/cloudinary-image';
import { RequiredFileHandlers } from '@comparaonline/ui-form-fields-next';
import { FieldNames } from '../constants';
import { FileController } from '../helpers';
import { AllowedPaymentMethod, CardButtonOptionItem, Offer } from '../types';
import { getFormattedCurrency } from './currency';
import { PaymentLogo } from '../components/PaymentLogo';

interface PaymentMethodOption
  extends Omit<CardButtonOptionItem, 'description' | 'secondaryLabel' | 'secondarySubLabel'> {
  description: (paymentMethod: AllowedPaymentMethod) => CardButtonOptionItem['description'];
  secondarySubLabel: (
    paymentMethod: AllowedPaymentMethod
  ) => CardButtonOptionItem['secondarySubLabel'];
}

const I18N_PREFIX = `${FieldNames.Payment.Method}.values`;

const paymentMethodOptions: Record<string, (offer: Offer) => PaymentMethodOption | undefined> = {
  [CicoPaymentMethod.CreditCard]: () => ({
    description: () => t => t(`${I18N_PREFIX}.${CicoPaymentMethod.CreditCard}.description`),
    image: <CloudinaryImage name="v1629823413/assets/payments/co/credit-card.svg" />,
    imageUnchecked: <CloudinaryImage name="v1629824152/assets/payments/co/credit-card_gray.svg" />,
    label: t => t(`${I18N_PREFIX}.${CicoPaymentMethod.CreditCard}.label`),
    secondarySubLabel: () => undefined,
    value: CicoPaymentMethod.CreditCard
  }),
  [CicoPaymentMethod.Financed]: () => ({
    description: ({ interest = 0 }) => t =>
      t(`${I18N_PREFIX}.${CicoPaymentMethod.Financed}.description`, { count: interest }),
    image: <CloudinaryImage name="v1629823413/assets/payments/co/finesa.svg" />,
    imageUnchecked: <CloudinaryImage name="v1629824152/assets/payments/co/finesa_gray.png" />,
    label: t => t(`${I18N_PREFIX}.${CicoPaymentMethod.Financed}.label`),
    secondarySubLabel: ({ installments }) => t =>
      t(`${I18N_PREFIX}.${CicoPaymentMethod.Financed}.secondarySubLabel`, { installments }),
    value: CicoPaymentMethod.Financed
  }),
  [CicoPaymentMethod.Pse]: () => ({
    description: () => t => t(`${I18N_PREFIX}.${CicoPaymentMethod.Pse}.description`),
    image: <CloudinaryImage name="v1629823413/assets/payments/co/pse.svg" />,
    imageUnchecked: <CloudinaryImage name="v1629910330/assets/payments/co/pse_gray.png" />,
    label: t => t(`${I18N_PREFIX}.${CicoPaymentMethod.Pse}.label`),
    secondarySubLabel: () => undefined,
    value: CicoPaymentMethod.Pse
  }),
  [CicoPaymentMethod.DirectToCompany]: ({
    product: {
      division: { logo }
    }
  }) => ({
    description: () => t => t(`${I18N_PREFIX}.${CicoPaymentMethod.DirectToCompany}.description`),
    image: <PaymentLogo logo={logo} />,
    imageUnchecked: <PaymentLogo logo={logo} isDisabled={true} />,
    label: t => t(`${I18N_PREFIX}.${CicoPaymentMethod.DirectToCompany}.label`),
    secondarySubLabel: () => undefined,
    value: CicoPaymentMethod.DirectToCompany
  })
};

export const getPaymentMethodOptions = (
  paymentMethods: AllowedPaymentMethod[],
  offer: Offer
): CardButtonOptionItem[] => {
  return paymentMethods
    .filter(({ name }) => Boolean(paymentMethodOptions[name]))
    .map(paymentMethod => {
      const { description, secondarySubLabel, ...option } = paymentMethodOptions[
        paymentMethod.name
      ](offer) as PaymentMethodOption;

      return {
        ...option,
        description: description(paymentMethod),
        secondaryLabel: () => getFormattedCurrency(paymentMethod.amount),
        secondarySubLabel: secondarySubLabel(paymentMethod)
      } as CardButtonOptionItem;
    });
};

export const isPaymentMethod = (formApi: FormApi, compareTo: CicoPaymentMethod): boolean => {
  const { values } = formApi.getState();
  const paymentMethod: CicoPaymentMethod = getIn(values, FieldNames.Payment.Method);
  return compareTo === paymentMethod;
};

const getPromissoryNote = (formApi: FormApi) => {
  const { values } = formApi.getState();
  const promissoryNote = getIn(values, FieldNames.Files.PromissoryNote);
  if (!promissoryNote || !promissoryNote.downloadUrl) return {};
  const { downloadUrl, fileName } = promissoryNote;
  return { downloadUrl, fileName };
};

export const downloadPromissoryNote = (
  formApi: FormApi,
  downloadFile: RequiredFileHandlers['downloadFile']
): void => {
  const { downloadUrl, fileName } = getPromissoryNote(formApi);
  if (!downloadUrl) return;
  downloadFile(fileName, downloadUrl);
};

export interface DownloadPromissoryNoteAutomaticallyProps {
  downloadFile: RequiredFileHandlers['downloadFile'];
  formApi: FormApi<AnyObject>;
  offerId: string;
  quoteId: string;
}

export const downloadPromissoryNoteAutomatically = ({
  formApi,
  downloadFile,
  offerId,
  quoteId
}: DownloadPromissoryNoteAutomaticallyProps): void => {
  const { getState, setState } = FileController(offerId, quoteId);
  const documentState = getState();
  const { downloadUrl, fileName } = getPromissoryNote(formApi);

  if (!downloadUrl || documentState[downloadUrl]) return;
  downloadFile(fileName, downloadUrl);
  setState(downloadUrl, true);
};
