import { FormApi } from 'final-form';
import { filterUndefinedData } from './filterUndefinedData';
import { SarlaftPersonType, Subcollection, Submitter, DeepPartial } from '../types';
import { transformOutput } from '../helpers';
import { isValidSarlaft } from './index';
import { Checkout, Inspection, Sarlaft } from '@comparaonline/cico-funnel-schema';

const INSPECTION_COLLECTION_NAME = 'inspection';
const SARLAFT_COLLECTION_NAME = 'sarlaft';

const getInspectionCollection = (data): Subcollection => {
  const values = data.inspection;
  return {
    name: INSPECTION_COLLECTION_NAME,
    values: {
      ...values,
      isCompleted: isInspectionCompleted(values)
    }
  };
};

const isInspectionCompleted = (inspection): boolean => Boolean(inspection.inspectionId);

const getSarlaftCollection = (
  data,
  sarlaftPersonType: SarlaftPersonType,
  formApi: FormApi
): Subcollection => {
  const sarlaftPersonTypeLower = sarlaftPersonType.toLowerCase();
  const values = data.sarlaft[sarlaftPersonTypeLower];

  return {
    id: sarlaftPersonTypeLower,
    name: SARLAFT_COLLECTION_NAME,
    values: {
      ...values,
      isCompleted: isValidSarlaft(sarlaftPersonType, formApi)
    }
  };
};

const prepareFirestoreData = (values, formApi: FormApi) => {
  const data = transformOutput(filterUndefinedData(values));
  const inspectionCollection = getInspectionCollection(data);
  const insuredCollection = getSarlaftCollection(data, SarlaftPersonType.Insured, formApi);
  const payerCollection = getSarlaftCollection(data, SarlaftPersonType.Payer, formApi);

  const collections: Subcollection[] = [
    inspectionCollection,
    insuredCollection,
    payerCollection
  ].filter((collection: Subcollection): boolean =>
    Boolean(collection.values && collection.values.isCompleted)
  );

  const fireStoreData: DeepPartial<Checkout> = {
    ...data,
    inspection: inspectionCollection.values as Inspection,
    sarlaft: {
      insured: insuredCollection.values as Sarlaft,
      payer: payerCollection.values as Sarlaft
    }
  };

  return { fireStoreData, collections };
};

const prepareFirestoreDataHandler = (submit: Submitter) => (values, formApi: FormApi): void => {
  const { fireStoreData, collections } = prepareFirestoreData(values, formApi);
  submit(fireStoreData, collections);
};

export { prepareFirestoreData, prepareFirestoreDataHandler };
