import { FormApi } from 'final-form';
import { CicoAutomasInspectionType } from '@comparaonline/inspection-scheduler-schema';
import {
  AlertSeverity,
  AlertWarning,
  CardButtonGroup,
  CardButtonVariant,
  TextFieldFilled
} from '@comparaonline/ui-form-fields-next';
import { Step } from '@comparaonline/ui-wizard-step-manager';
import { FormatMessage } from '@comparaonline/react-intl-hooks';
import { validateWith } from '@comparaonline/ui-forms-validators/dist/validateWith';
import { ColombiaPhoneInputMask } from '@comparaonline/ui-text-masks';

import { StepNames, StepIds, FieldNames, NoFieldNames } from '../constants';
import { ButtonInspectionCenter } from '../components/ButtonInspectionCenter';
import { CardGeneratorField } from '../components/CardGenerator';
import { NumberInspectionOrder } from '../components/NumberInspectionOrder';
import { TypographyFiled } from '../components/TypographyFiled';
import { VirtualInspection } from '../components';
import {
  getInspectionOptions,
  hasErrorInspectionOrder,
  hasOrderNumber,
  isCreateOrderWithPresential,
  isVirtualMode,
  showForVirtualMode
} from '../helpers';
import { AllowedInspectionType, Submitter } from '../types';
import * as validators from '../validators';

export const inspectionAutomasStep = (
  createInspection: Submitter,
  showInspectionCenter: () => void,
  allowedInspectionTypes: AllowedInspectionType
): Step => ({
  section: {
    id: 'inspection-automas-section',
    name: () => StepNames.Inspection
  },
  id: StepIds.InspectionAutomas,
  next: StepIds.CarOwner,
  fields: [
    {
      name: NoFieldNames.Inspection.TitleCardButton,
      component: TypographyFiled,
      keyText: `${NoFieldNames.Inspection.TitleCardButton}.title`
    },
    {
      name: FieldNames.Inspection.Type,
      component: CardButtonGroup,
      variant: CardButtonVariant.Horizontal,
      options: getInspectionOptions(allowedInspectionTypes),
      warning: [
        {
          component: AlertWarning,
          text: t => t(`${FieldNames.Inspection.Type}.${CicoAutomasInspectionType.CEN}.alert`),
          severity: AlertSeverity.Info,
          when: (formApi: FormApi) => !isVirtualMode(formApi)
        },
        {
          component: AlertWarning,
          text: t =>
            t(`${FieldNames.Inspection.Type}.${CicoAutomasInspectionType.CEN}.alert.change`),
          severity: AlertSeverity.Warning,
          when: (formApi: FormApi) => isCreateOrderWithPresential(formApi)
        }
      ],
      validate: validateWith(validators.required),
      conditionToDisable: formApi => isCreateOrderWithPresential(formApi)
    },
    {
      component: VirtualInspection,
      conditionToRender: formApi => showForVirtualMode(formApi, allowedInspectionTypes),
      name: NoFieldNames.Inspection.VirtualInspectionModal,
      texts: {
        label: (t: FormatMessage) => t(`${NoFieldNames.Inspection.VirtualInspectionModal}.label`)
      },
      showLabel: true
    },
    {
      name: NoFieldNames.Inspection.TitleVirtualInputs,
      component: TypographyFiled,
      keyText: `${NoFieldNames.Inspection.TitleVirtualInputs}.title`,
      conditionToRender: formApi => showForVirtualMode(formApi, allowedInspectionTypes)
    },
    {
      component: TextFieldFilled,
      name: FieldNames.Inspection.Name,
      showLabel: true,
      texts: {
        label: (t: FormatMessage) => t(`${FieldNames.Inspection.Name}.label`)
      },
      conditionToRender: formApi => showForVirtualMode(formApi, allowedInspectionTypes)
    },
    {
      component: TextFieldFilled,
      name: FieldNames.Inspection.Phone,
      showLabel: true,
      texts: {
        label: (t: FormatMessage) => t(`${FieldNames.Inspection.Phone}.label`),
        placeholder: (t: FormatMessage) => t(`${FieldNames.Inspection.Phone}.placeholder`)
      },
      muiInputProps: {
        InputProps: {
          inputComponent: ColombiaPhoneInputMask,
          inputProps: {
            type: 'tel'
          }
        }
      },
      conditionToRender: formApi => showForVirtualMode(formApi, allowedInspectionTypes),
      validate: validateWith(validators.required, validators.validateMobilePhone)
    },
    {
      component: CardGeneratorField,
      conditionToRender: formApi => !hasOrderNumber(formApi),
      name: NoFieldNames.Inspection.InspectionOrder,
      onClick: ({ formApi }) => {
        const { values } = formApi.getState();
        formApi.change(FieldNames.FieldsNoSave.ErrorInspectionOrder, '');
        createInspection(values, formApi);
      },
      showLabel: true,
      texts: {
        buttonText: (t: FormatMessage) =>
          t(`${NoFieldNames.Inspection.InspectionOrder}.buttonText`),
        label: (t: FormatMessage) => t(`${NoFieldNames.Inspection.InspectionOrder}.label`),
        loading: (t: FormatMessage) => t(`${NoFieldNames.Inspection.InspectionOrder}.loading`)
      },
      validate: validateWith(validators.generatorOrder),
      warning: {
        component: AlertWarning,
        text: t => t(`${NoFieldNames.Inspection.InspectionOrder}.error.generate`),
        severity: AlertSeverity.Info,
        when: (formApi: FormApi) => hasErrorInspectionOrder(formApi)
      }
    },
    {
      name: NoFieldNames.Inspection.NumberOrder,
      component: NumberInspectionOrder,
      conditionToRender: formApi => hasOrderNumber(formApi)
    },
    {
      name: NoFieldNames.Inspection.InspectionCenter,
      component: ButtonInspectionCenter,
      showInspectionCenter,
      conditionToRender: formApi => !isVirtualMode(formApi),
      texts: {
        buttonText: (t: FormatMessage) => t(`${NoFieldNames.Inspection.InspectionCenter}.btn`)
      }
    }
  ]
});
