import { getIn } from 'final-form';
import { CicoPaymentMethod } from '@comparaonline/cico-funnel-schema';
import { Step } from '@comparaonline/ui-wizard-step-manager';
import {
  AlertSeverity,
  AlertField,
  AlertWarning,
  RadioGroupField
} from '@comparaonline/ui-form-fields-next';
import { validateWith } from '@comparaonline/ui-forms-validators/dist/validateWith';
import {
  ButtonLoading,
  CodeInput,
  Countdown,
  GoToChallengeQuestionFlow,
  ResendVerificationCode
} from '../components';
import {
  QUESTION_FIELDNAME_PREFIX,
  FieldNames,
  StepIds,
  identityValidationSection
} from '../constants';
import {
  canEnableQuestionsFlowSubmitButton,
  canGoToChallengeQuestionFlow,
  canResendCode,
  getRemainingAttemptsToValidate,
  isFailureIdentityValidation,
  isTimeExpired,
  isPaymentMethod,
  hasRemainingAttemptsToValidate
} from '../helpers';
import * as digitalSignatureService from '../helpers/digitalSignatureService';
import * as validators from '../validators';
import { GetDigitalSignatureStepsProps, OnGetQuestions, Question } from '../types';

export const getDigitalSignatureSteps = ({
  BFFURL,
  additionalDocumentName,
  onGetQuestions,
  questions,
  sarlaftSignatureRequired = true
}: GetDigitalSignatureStepsProps): Step[] => [
  {
    disableSubmitButton: formApi => hasRemainingAttemptsToValidate(formApi),
    fields: getValidationCodeFields(
      BFFURL,
      sarlaftSignatureRequired,
      onGetQuestions,
      additionalDocumentName
    ),
    id: StepIds.IdentityValidationCode,
    next: StepIds.Files,
    previous: StepIds.Files,
    section: identityValidationSection,
    subtitle: (t, values) => {
      const phone = getIn(values, FieldNames.Insured.Person.Phone);
      return t('identityValidationCode.subtitle', { phone });
    },
    title: t => t('identityValidationCode.title')
  },
  {
    disableSubmitButton: formApi => !canEnableQuestionsFlowSubmitButton(formApi),
    fields: getChallengeQuestionFields(
      BFFURL,
      sarlaftSignatureRequired,
      questions,
      additionalDocumentName
    ),
    id: StepIds.IdentityValidationChallengeQuestions,
    next: StepIds.Files,
    previous: StepIds.Files,
    section: identityValidationSection,
    subtitle: (t, values) => {
      const phone = getIn(values, FieldNames.Insured.Person.Phone);
      return t('identityValidation.questionsSubtitle', { phone });
    },
    title: t => t('identityValidation.questionsTitle')
  }
];

function getValidationCodeFields(
  BFFURL: string,
  sarlaftSignatureRequired: boolean,
  onGetQuestions: OnGetQuestions,
  additionalDocumentName?: string
) {
  return [
    {
      component: CodeInput,
      name: FieldNames.IdentityValidation.Code,
      showLabel: true,
      texts: {
        errorMessage: (t, _, formApi) =>
          t(`${FieldNames.IdentityValidation.Code}.attemptsRemainingError`, {
            attemptsRemaining: getRemainingAttemptsToValidate(formApi)
          })
      },
      validate: validateWith(validators.required)
    },
    {
      component: Countdown,
      name: FieldNames.IdentityValidation.ExpirationDate,
      warning: {
        component: AlertWarning,
        text: t => t(`${FieldNames.IdentityValidation.Code}.lastFailedAttempt`),
        severity: AlertSeverity.Warning,
        when: formApi => !hasRemainingAttemptsToValidate(formApi)
      }
    },
    {
      component: ResendVerificationCode,
      conditionToRender: formApi =>
        isTimeExpired(formApi) && canResendCode(formApi) && hasRemainingAttemptsToValidate(formApi),
      name: FieldNames.IdentityValidation.ResendCode,
      onClick: async ({ formApi }) => digitalSignatureService.reSendCode(formApi, BFFURL),
      showLabel: true,
      texts: {
        label: t => t('identityValidation.verificationCode.resendcode.label'),
        buttonText: t => t('identityValidation.verificationCode.resendcode.button'),
        helperText: t => t('identityValidation.verificationCode.resendcode.helperText')
      }
    },
    {
      component: GoToChallengeQuestionFlow,
      conditionToRender: canGoToChallengeQuestionFlow,
      warning: {
        component: AlertWarning,
        text: t => t(`${FieldNames.IdentityValidation.ExpirationDate}.useAnotherMethod`),
        severity: AlertSeverity.Info,
        when: formApi => !canResendCode(formApi) && isTimeExpired(formApi)
      },
      onClick: async ({ formApi, stepManagerApi }) => {
        const response = await digitalSignatureService.bypassCodeValidation(formApi, BFFURL);
        if (response.ok) {
          stepManagerApi.navigationApi.goTo(StepIds.IdentityValidationChallengeQuestions);
          if (response.questions) {
            onGetQuestions(response.questions);
          }
        }
      },
      texts: {
        buttonText: t => t('identityValidation.verificationCode.goToChallengeQuestionFlow.label')
      }
    },
    {
      component: ButtonLoading,
      conditionToRender: formApi => {
        const remainingAttemptsFailed = !hasRemainingAttemptsToValidate(formApi);
        const cannotResendCode = !canResendCode(formApi) && isTimeExpired(formApi);

        return remainingAttemptsFailed || cannotResendCode ? false : true;
      },
      name: FieldNames.FieldsNoSave.ValidateCode,
      onClick: async ({ formApi, stepManagerApi }) => {
        const isSigned = await digitalSignatureService.verifyCodeAndSign(
          formApi,
          BFFURL,
          sarlaftSignatureRequired,
          additionalDocumentName
        );
        if (isSigned) {
          stepManagerApi.navigationApi.goTo(StepIds.Files);
        }
      },
      texts: {
        buttonText: t => t(`${FieldNames.FieldsNoSave.ValidateCode}.button`),
        loadingText: t => t(`${FieldNames.FieldsNoSave.ValidateCode}.loading`)
      }
    }
  ];
}

function getChallengeQuestionFields(
  BFFURL: string,
  sarlaftSignatureRequired: boolean,
  questions?: Question[],
  additionalDocumentName?: string
) {
  if (!questions) return;

  const fields = [
    {
      conditionToRender: formApi => !isTimeExpired(formApi),
      component: AlertField,
      severity: AlertSeverity.Info,
      showLabel: true,
      texts: {
        label: t => t(`identityValidation.questionsInfoWarning`)
      }
    },
    {
      component: Countdown,
      name: FieldNames.IdentityValidation.ExpirationDate,
      warning: {
        component: AlertWarning,
        text: (t, _values, formApi) =>
          t(`${FieldNames.IdentityValidation.ExpirationDate}.expiredSignatureText`, {
            hasPromissoryNote: isPaymentMethod(formApi, CicoPaymentMethod.Financed)
          }),
        severity: AlertSeverity.Warning,
        when: formApi => isTimeExpired(formApi)
      }
    },
    ...questions.map(({ answers, ...question }) => {
      const field = {
        name: `${FieldNames.IdentityValidation.IdentityValidationAnswers}.${QUESTION_FIELDNAME_PREFIX}${question.value}`,
        component: RadioGroupField,
        texts: {
          label: () => question.label
        },
        options: answers?.map(({ value, label }) => ({
          value,
          label: () => label
        })),
        validate: validateWith(validators.required),
        showLabel: true
      };

      return field;
    }),
    {
      conditionToRender: formApi => !isFailureIdentityValidation(formApi),
      component: ButtonLoading,
      name: FieldNames.FieldsNoSave.ValidateQuestions,
      texts: {
        buttonText: t => t(`${FieldNames.FieldsNoSave.ValidateQuestions}.button`),
        loadingText: t => t(`${FieldNames.FieldsNoSave.ValidateQuestions}.loading`)
      },
      onClick: async ({ formApi, stepManagerApi }) => {
        formApi.submit();
        const { errors } = formApi.getState();
        if (errors.identityValidation) {
          return false;
        }
        const isSigned = await digitalSignatureService.verifyAnswersAndSign(
          formApi,
          BFFURL,
          sarlaftSignatureRequired,
          additionalDocumentName
        );
        if (isSigned) {
          stepManagerApi.navigationApi.goTo(StepIds.Files);
        }
      }
    },
    {
      conditionToRender: isFailureIdentityValidation,
      component: AlertField,
      showLabel: true,
      texts: {
        label: t => t('identityValidation.challengeQuestions.failWarning')
      }
    }
  ];

  return fields;
}
