import { FormApi, getIn } from 'final-form';
import { CicoPaymentMethod } from '@comparaonline/cico-funnel-schema';
import { DividerFieldContainer, Step } from '@comparaonline/ui-wizard-step-manager';
import { SectionProps } from '@comparaonline/ui-wizard-step-manager/dist/types';
import {
  AlertSeverity,
  AlertWarning,
  ButtonType,
  InputButton,
  RequiredFileHandlers,
  TextFieldFilled,
  Uploader,
  UploaderMultiple
} from '@comparaonline/ui-form-fields-next';
import { validateWith } from '@comparaonline/ui-forms-validators/dist/validateWith';

import {
  DigitalSignature,
  DocumentsLoading,
  ButtonLoading,
  DocumentValidationResult
} from '../components';
import {
  FieldNames,
  identificationDocumentVersionOptions,
  StatusPromissoryNote,
  StepIds,
  StepNames
} from '../constants';
import {
  canEnableFileStepSubmitButton,
  canInitializeIdentityValidation,
  fieldValueIsFalse,
  isFailureIdentityValidation,
  isPaymentMethod,
  isValidSarlaft,
  canRenderDigitalSignatureField,
  canRenderSignButton,
  hasPromissoryNote,
  signedInsuredSarlaftUploaded,
  copyInsuredSarlaftDownloadUrlToUploadUrl,
  insuredSarlaftGenerated,
  getPromissoryNote,
  getPromissoryNoteRetries,
  getPromissoryNoteStatus,
  displayDigitalSignatureForPromissoryNote,
  displayManualUploadForPromissoryNote,
  validateDocuments,
  canValidateDocuments,
  shouldDisplayIdentificationDocumentVersionSelector,
  shouldDisplayBackOfIdentificationDocumentInput
} from '../helpers';
import { OnGetQuestions, SarlaftPersonType } from '../types';
import * as validators from '../validators';
import * as digitalSignatureService from '../helpers/digitalSignatureService';
import { generator } from '../validators';
import { additionalSarlaftDocValidator } from '../validators/additionalSarlaftDoc';
import { CicoForm } from '@comparaonline/quote-forms-schema/dist/schema';

const fileSection: SectionProps = {
  id: 'file-section',
  name: () => StepNames.File
};

const inputFileAccept = 'image/*,.pdf';

const requirePayerSarlaft = (formApi: FormApi): boolean =>
  fieldValueIsFalse(FieldNames.Insured.IsOwnerThePayer)(formApi);

type GetFileStep = (props: {
  additionalDocumentFilename: string;
  bffURL: string;
  fulfillmentPresenterURL: string;
  fileHandler: RequiredFileHandlers;
  isDigitalSignatureEnabled: boolean;
  sarlaftSignatureRequired?: boolean;
  offerId: string;
  onGetQuestions: OnGetQuestions;
  quoteId: string;
  showAdditionalDocument: boolean;
  previousStep: StepIds;
  quoteForm: CicoForm;
}) => Step[];

const getFileSteps: GetFileStep = ({
  additionalDocumentFilename,
  bffURL,
  fulfillmentPresenterURL,
  fileHandler,
  isDigitalSignatureEnabled,
  sarlaftSignatureRequired = true,
  offerId,
  onGetQuestions,
  quoteId,
  showAdditionalDocument,
  previousStep
}) => [
  {
    section: fileSection,
    id: StepIds.Files,
    previous: previousStep,
    subtitle: t => t('files.subtitle'),
    title: t => t('files.title'),
    disableSubmitButton: formApi => canEnableFileStepSubmitButton(formApi),
    onStepLoad: ({ formApi }) => {
      if (isPaymentMethod(formApi, CicoPaymentMethod.Financed)) {
        getPromissoryNote(formApi);
      }
      if (
        !sarlaftSignatureRequired &&
        insuredSarlaftGenerated(formApi) &&
        !signedInsuredSarlaftUploaded(formApi)
      ) {
        copyInsuredSarlaftDownloadUrlToUploadUrl(formApi);
      }
    },
    fields: [
      {
        ...fileHandler,
        additionalDocumentFilename,
        showAdditionalDocument,
        component: DocumentsLoading,
        cicoMetadata: { offerId, quoteId },
        sarlaftSignatureRequired,
        autoDownload: formApi =>
          !canInitializeIdentityValidation(formApi, isDigitalSignatureEnabled),
        warning: [
          {
            component: AlertWarning,
            text: (t, _, formApi) =>
              t(`${FieldNames.Files.InsuredSarlaft}.failedSignatureWarningText`, {
                hasPromissoryNote: isPaymentMethod(formApi, CicoPaymentMethod.Financed)
              }),
            severity: AlertSeverity.Warning,
            when: isFailureIdentityValidation
          },
          {
            component: AlertWarning,
            text: (t, _) => t(`${FieldNames.Files.PromissoryNote}.retryPromissoryNote`),
            severity: AlertSeverity.Error,
            when: formApi => {
              const promissoryNoteStatus = getPromissoryNoteStatus(formApi);
              const hasRetries = getPromissoryNoteRetries(formApi) === 0;
              return (
                !hasPromissoryNote(formApi) &&
                hasRetries &&
                promissoryNoteStatus === StatusPromissoryNote.Failed
              );
            }
          },
          {
            component: AlertWarning,
            text: (t, _) => t(`${FieldNames.Files.PromissoryNote}.failedPromissoryNote`),
            severity: AlertSeverity.Info,
            when: formApi => {
              const promissoryNoteStatus = getPromissoryNoteStatus(formApi);
              const hasRetries = getPromissoryNoteRetries(formApi) === 0;
              return (
                !hasPromissoryNote(formApi) &&
                !hasRetries &&
                promissoryNoteStatus === StatusPromissoryNote.Failed
              );
            }
          }
        ]
      },
      {
        component: ButtonLoading,
        conditionToRender: formApi => {
          const promissoryNoteStatus = getPromissoryNoteStatus(formApi);
          const hasRetries = getPromissoryNoteRetries(formApi) === 0;
          return (
            !hasPromissoryNote(formApi) &&
            hasRetries &&
            promissoryNoteStatus === StatusPromissoryNote.Failed
          );
        },
        onClick: async ({ formApi }) => {
          formApi.change(FieldNames.PromissoryNote.Retries, 1);
          getPromissoryNote(formApi);
        },
        texts: {
          buttonText: t => t(`${FieldNames.Files.PromissoryNote}.error.buttonText`),
          loadingText: t => t(`${FieldNames.Files.PromissoryNote}.error.loadingText`)
        }
      },
      {
        component: Uploader,
        conditionToRender: formApi =>
          !canInitializeIdentityValidation(formApi, isDigitalSignatureEnabled),
        inputFileAccept,
        name: FieldNames.Files.InsuredSarlaft,
        container: DividerFieldContainer,
        texts: {
          helperText: (t, _values, formApi) =>
            !sarlaftSignatureRequired && signedInsuredSarlaftUploaded(formApi)
              ? t(`${FieldNames.Files.InsuredSarlaft}.signatureHelperTextSignatureNotRequired`)
              : t(`${FieldNames.Files.InsuredSarlaft}.helperText`),
          label: t => t(`${FieldNames.Files.InsuredSarlaft}.label`),
          buttonText: t => t(`${FieldNames.Files.InsuredSarlaft}.buttonText`),
          buttonTextAfter: t => t(`${FieldNames.Files.InsuredSarlaft}.buttonTextAfter`),
          secondaryButtonText: t => t(`${FieldNames.Files.InsuredSarlaft}.secondaryButtonText`)
        },
        showLabel: true,
        validate: validateWith(validators.requiredFile),
        ...fileHandler
      },
      {
        ...fileHandler,
        component: Uploader,
        conditionToRender: formApi =>
          !canInitializeIdentityValidation(formApi, isDigitalSignatureEnabled) &&
          showAdditionalDocument,
        inputFileAccept,
        name: FieldNames.Files.SignedAdditionalDocument,
        container: DividerFieldContainer,
        texts: {
          helperText: t => t(`${FieldNames.Files.AdditionalDocument}.helperText`),
          label: () => additionalDocumentFilename,
          buttonText: t => t(`${FieldNames.Files.AdditionalDocument}.buttonText`),
          buttonTextAfter: t => t(`${FieldNames.Files.AdditionalDocument}.buttonTextAfter`),
          secondaryButtonText: t =>
            t(`${FieldNames.Files.AdditionalDocument}.secondaryButtonText`, {
              filename: additionalDocumentFilename
            })
        },
        getFileInfo: ({ formApi, buttonType }) => {
          const { values } = formApi.getState();
          const documentUrl =
            buttonType === ButtonType.Main
              ? getIn(values, FieldNames.Files.SignedAdditionalDocument)
              : getIn(values, FieldNames.Files.AdditionalDocument);
          return {
            fileName: additionalDocumentFilename,
            url: documentUrl
          };
        },
        customFileName: additionalDocumentFilename,
        customUploadDateField: FieldNames.Files.SignedAdditionalDocumentUploadDate,
        showLabel: true,
        validate: validateWith(additionalSarlaftDocValidator),
        saveFile: async (...args) => {
          // @ts-ignore
          const saveFileResult = await fileHandler.saveFile(...args);
          return saveFileResult.uploadUrl;
        },
        deleteFile: (_value, formApi) => {
          formApi.change(FieldNames.Files.SignedAdditionalDocument, undefined);
          formApi.change(FieldNames.Files.SignedAdditionalDocumentUploadDate, undefined);
        }
      },
      {
        conditionToRender: formApi =>
          canRenderDigitalSignatureField(
            formApi,
            isDigitalSignatureEnabled,
            `${FieldNames.Files.InsuredSarlaft}.downloadUrl`
          ),
        container: DividerFieldContainer,
        name: FieldNames.Files.InsuredSarlaft,
        component: DigitalSignature,
        texts: {
          helperText: t =>
            sarlaftSignatureRequired
              ? t(`${FieldNames.Files.InsuredSarlaft}.signatureHelperText`)
              : t(`${FieldNames.Files.InsuredSarlaft}.signatureHelperTextSignatureNotRequired`),
          documentButtonText: t => t(`${FieldNames.Files.InsuredSarlaft}.documentButtonText`),
          buttonEdit: t => t(`${FieldNames.Files.InsuredSarlaft}.editButtonText`),
          fileName: t => t(`${FieldNames.Files.InsuredSarlaft}.fileName`),
          title: t => t(`${FieldNames.Files.InsuredSarlaft}.modal.title`),
          subtitle: t => t(`${FieldNames.Files.InsuredSarlaft}.modal.subtitle`)
        },
        showLabel: true,
        validate: validateWith(validators.requiredFile),
        ...fileHandler
      },
      {
        ...fileHandler,
        component: DigitalSignature,
        conditionToRender: formApi =>
          canRenderDigitalSignatureField(
            formApi,
            isDigitalSignatureEnabled && showAdditionalDocument,
            FieldNames.Files.AdditionalDocument
          ),
        container: DividerFieldContainer,
        getFileInfo: ({ formApi }) => {
          const { values } = formApi.getState();
          const signedAdditionalDocumentUrl = getIn(
            values,
            FieldNames.Files.SignedAdditionalDocument
          );
          return {
            fileName: additionalDocumentFilename,
            url: signedAdditionalDocumentUrl
          };
        },
        name: FieldNames.Files.AdditionalDocument,
        nameSufix: '',
        signedDocPath: `${FieldNames.Files.InsuredSarlaft}.additionalSignedDocumentUrl`,
        texts: {
          documentButtonText: t => t(`${FieldNames.Files.AdditionalDocument}.documentButtonText`),
          helperText: t =>
            t(`${FieldNames.Files.AdditionalDocument}.signatureHelperText`, {
              filename: additionalDocumentFilename
            }),
          fileName: t =>
            t(`${FieldNames.Files.AdditionalDocument}.fileName`, {
              filename: additionalDocumentFilename
            }),
          title: t =>
            t(`${FieldNames.Files.AdditionalDocument}.modal.title`, {
              filename: additionalDocumentFilename
            }),
          subtitle: t => t(`${FieldNames.Files.AdditionalDocument}.modal.subtitle`)
        },
        showLabel: true,
        validate: validateWith(validators.requiredFile)
      },
      {
        conditionToRender: formApi =>
          displayDigitalSignatureForPromissoryNote(formApi) &&
          canInitializeIdentityValidation(formApi, isDigitalSignatureEnabled),
        container: DividerFieldContainer,
        name: FieldNames.Files.PromissoryNote,
        component: DigitalSignature,
        texts: {
          helperText: t => t(`${FieldNames.Files.PromissoryNote}.signatureHelperText`),
          documentButtonText: t => t(`${FieldNames.Files.PromissoryNote}.documentButtonText`),
          buttonEdit: t => t(`${FieldNames.Files.PromissoryNote}.editButtonText`),
          fileName: t => t(`${FieldNames.Files.PromissoryNote}.fileName`),
          title: t => t(`${FieldNames.Files.PromissoryNote}.modal.title`),
          subtitle: t => t(`${FieldNames.Files.PromissoryNote}.modal.subtitle`)
        },
        showLabel: true,
        validate: validateWith(
          generator({
            errorCode: `${FieldNames.Files.PromissoryNote}.error`,
            valueName: `${FieldNames.Files.PromissoryNote}.downloadUrl`
          })
        ),
        ...fileHandler
      },
      {
        component: ButtonLoading,
        conditionToRender: formApi =>
          canRenderSignButton(formApi, isDigitalSignatureEnabled, showAdditionalDocument),
        onClick: async ({ formApi, stepManagerApi }) => {
          const response = await digitalSignatureService.initIdentityValidation(
            formApi,
            bffURL,
            quoteId
          );

          if (response.ok) {
            stepManagerApi.navigationApi.goTo(response.step);
            if (response.questions) {
              onGetQuestions(response.questions);
            }
          }
        },
        texts: {
          buttonText: (t, _values, formApi) =>
            !sarlaftSignatureRequired && hasPromissoryNote(formApi)
              ? t('identityValidation.buttonSignPromissoryNote')
              : t('identityValidation.buttonSignAll'),
          loadingText: t => t('identityValidation.loadingSign')
        },
        warning: {
          component: AlertWarning,
          text: t => t(`${FieldNames.Files.InsuredSarlaft}.infoWarningText`),
          severity: AlertSeverity.Info
        }
      },
      {
        component: Uploader,
        conditionToRender: formApi =>
          displayManualUploadForPromissoryNote(formApi) &&
          !canInitializeIdentityValidation(formApi, isDigitalSignatureEnabled),
        inputFileAccept: 'image/*,.pdf',
        name: FieldNames.Files.PromissoryNote,
        showLabel: true,
        texts: {
          helperText: t => t(`${FieldNames.Files.PromissoryNote}.helperText`),
          label: t => t(`${FieldNames.Files.PromissoryNote}.label`),
          buttonText: t => t(`${FieldNames.Files.PromissoryNote}.buttonText`),
          buttonTextAfter: t => t(`${FieldNames.Files.PromissoryNote}.buttonTextAfter`),
          secondaryButtonText: t => t(`${FieldNames.Files.PromissoryNote}.secondaryButtonText`)
        },
        validate: validateWith(validators.requiredFile),
        ...fileHandler
      },
      {
        component: InputButton,
        container: DividerFieldContainer,
        conditionToRender: (formApi: FormApi) =>
          !isValidSarlaft(SarlaftPersonType.Payer, formApi) && requirePayerSarlaft(formApi),
        showLabel: true,
        texts: {
          label: t => t('files.sarlaftPayer.label'),
          callToAction: t => t('files.sarlaftPayer.callToAction'),
          helperText: t => t('files.sarlaftPayer.helperText')
        },
        onClick: ({ stepManagerApi: { navigationApi } }) => navigationApi.goTo(StepIds.Payer)
      },
      {
        component: Uploader,
        inputFileAccept,
        container: DividerFieldContainer,
        name: FieldNames.Files.PayerSarlaft,
        conditionToRender: (formApi: FormApi) =>
          isValidSarlaft(SarlaftPersonType.Payer, formApi) && requirePayerSarlaft(formApi),
        showLabel: true,
        texts: {
          helperText: t => t(`${FieldNames.Files.PayerSarlaft}.helperText`),
          label: t => t(`${FieldNames.Files.PayerSarlaft}.label`),
          buttonText: t => t(`${FieldNames.Files.PayerSarlaft}.buttonText`),
          buttonTextAfter: t => t(`${FieldNames.Files.PayerSarlaft}.buttonTextAfter`),
          secondaryButtonText: t => t(`${FieldNames.Files.InsuredSarlaft}.secondaryButtonText`)
        },
        validate: validateWith(validators.requiredFile),
        ...fileHandler
      },
      {
        component: TextFieldFilled,
        name: FieldNames.IdentificationDocumentVersion,
        options: identificationDocumentVersionOptions,
        select: true,
        showLabel: true,
        texts: {
          label: t => t(`${FieldNames.IdentificationDocumentVersion}.label`)
        },
        validate: validateWith(validators.required),
        conditionToRender: (formApi: FormApi) =>
          shouldDisplayIdentificationDocumentVersionSelector(formApi)
      },
      {
        component: Uploader,
        inputFileAccept,
        name: FieldNames.Files.IdentificationDocument,
        container: DividerFieldContainer,
        texts: {
          helperText: t => t(`${FieldNames.Files.IdentificationDocument}.helperText`),
          label: (t, _values, formApi) =>
            canValidateDocuments(formApi)
              ? t(`${FieldNames.Files.IdentificationDocument}.labelFront`)
              : t(`${FieldNames.Files.IdentificationDocument}.label`),
          buttonText: t => t(`${FieldNames.Files.IdentificationDocument}.buttonText`),
          buttonTextAfter: t => t(`${FieldNames.Files.IdentificationDocument}.buttonTextAfter`)
        },
        showLabel: true,
        validate: validateWith(validators.requiredFile),
        ...fileHandler
      },
      {
        component: Uploader,
        inputFileAccept,
        conditionToRender: (formApi: FormApi) =>
          shouldDisplayBackOfIdentificationDocumentInput(formApi),
        name: FieldNames.Files.IdentificationDocumentBack,
        container: DividerFieldContainer,
        texts: {
          helperText: t => t(`${FieldNames.Files.IdentificationDocumentBack}.helperText`),
          label: t => t(`${FieldNames.Files.IdentificationDocumentBack}.label`),
          buttonText: t => t(`${FieldNames.Files.IdentificationDocumentBack}.buttonText`),
          buttonTextAfter: t => t(`${FieldNames.Files.IdentificationDocumentBack}.buttonTextAfter`)
        },
        showLabel: true,
        validate: validateWith(validators.requiredFile),
        ...fileHandler
      },
      {
        component: Uploader,
        inputFileAccept,
        name: FieldNames.Files.PropertyCertificate,
        container: DividerFieldContainer,
        texts: {
          helperText: t => t(`${FieldNames.Files.PropertyCertificate}.helperText`),
          label: (t, _values, formApi) =>
            canValidateDocuments(formApi)
              ? t(`${FieldNames.Files.PropertyCertificate}.labelFront`)
              : t(`${FieldNames.Files.PropertyCertificate}.label`),
          buttonText: t => t(`${FieldNames.Files.PropertyCertificate}.buttonText`),
          buttonTextAfter: t => t(`${FieldNames.Files.PropertyCertificate}.buttonTextAfter`)
        },
        showLabel: true,
        validate: validateWith(validators.requiredFile),
        ...fileHandler
      },
      {
        component: Uploader,
        inputFileAccept,
        conditionToRender: (formApi: FormApi) => canValidateDocuments(formApi),
        name: FieldNames.Files.PropertyCertificateBack,
        container: DividerFieldContainer,
        texts: {
          helperText: t => t(`${FieldNames.Files.PropertyCertificateBack}.helperText`),
          label: t => t(`${FieldNames.Files.PropertyCertificateBack}.label`),
          buttonText: t => t(`${FieldNames.Files.PropertyCertificateBack}.buttonText`),
          buttonTextAfter: t => t(`${FieldNames.Files.PropertyCertificateBack}.buttonTextAfter`)
        },
        showLabel: true,
        validate: validateWith(validators.requiredFile),
        ...fileHandler
      },
      {
        component: ButtonLoading,
        conditionToRender: (formApi: FormApi) => canValidateDocuments(formApi),
        name: FieldNames.FieldsNoSave.ValidateDocuments,
        onClick: async ({ formApi }) => {
          await validateDocuments(formApi, quoteId, fulfillmentPresenterURL);
        },
        texts: {
          buttonText: t => t(`${FieldNames.FieldsNoSave.ValidateDocuments}.button`),
          loadingText: t => t(`${FieldNames.FieldsNoSave.ValidateDocuments}.loading`)
        }
      },
      {
        component: DocumentValidationResult,
        conditionToRender: (formApi: FormApi) => {
          const { values } = formApi.getState();
          const documentsValidationResult = getIn(
            values,
            FieldNames.DocumentsValidationResult.Result
          );
          return !!documentsValidationResult;
        }
      },
      {
        component: UploaderMultiple,
        inputFileAccept,
        name: FieldNames.Files.OtherDocuments,
        texts: {
          label: t => t(`${FieldNames.Files.OtherDocuments}.label`),
          buttonText: t => t(`${FieldNames.Files.OtherDocuments}.buttonText`),
          buttonTextAfter: t => t(`${FieldNames.Files.OtherDocuments}.buttonTextAfter`)
        },
        showLabel: true,
        ...fileHandler
      }
    ]
  }
];

export { fileSection, getFileSteps };
