import { TextFieldFilled } from '@comparaonline/ui-form-fields-next';
import { IFieldProps, Step } from '@comparaonline/ui-wizard-step-manager';
import { validateWith } from '@comparaonline/ui-forms-validators/dist/validateWith';
import { InputModeProps } from '@comparaonline/ui-input-helpers';

import { MoneyFormat } from '../components';
import {
  FieldNames,
  MAX_MONEYFIELD_LENGTH,
  StepIds,
  insuredSection,
  payerSection
} from '../constants';
import { fieldIsRequired, fieldParser } from '../helpers';
import { SarlaftPersonType } from '../types';
import * as validators from '../validators';
import { getPrincipalSourceOfIncomeField } from './fields/principalSourceOfIncome';
import { bypassValidator } from '../validators';

const getAssetFieldConfig = (
  sarlaftPersonType: SarlaftPersonType,
  carValuation: string,
  isRequired: boolean
): Partial<IFieldProps> => {
  const isInsuredSarlaft = sarlaftPersonType === SarlaftPersonType.Insured;
  const min = isInsuredSarlaft ? carValuation : undefined;
  const validate = isInsuredSarlaft
    ? validateWith(
        isRequired ? validators.required : bypassValidator,
        validators.minNumber({
          min: Number(carValuation.replace(/\./g, '')),
          variables: { min: Number(carValuation).toLocaleString('es-co') }
        })
      )
    : isRequired && validateWith(validators.required);

  return {
    muiInputProps: {
      InputProps: {
        inputProps: {
          min
        }
      }
    },
    validate
  };
};

const financialSteps = (
  sarlaftPersonType: SarlaftPersonType,
  carValuation: string,
  optionalFieldNames?: string[]
): Step[] => {
  const { Work } = FieldNames[sarlaftPersonType];
  const isAssetRequired = fieldIsRequired(Work.Assets, optionalFieldNames);
  const assetsFieldConfig = getAssetFieldConfig(sarlaftPersonType, carValuation, isAssetRequired);

  return [
    {
      section: sarlaftPersonType === SarlaftPersonType.Insured ? insuredSection : payerSection,
      id: StepIds[`${sarlaftPersonType}Financial`],
      next: StepIds[`${sarlaftPersonType}SourceOfIncome`],
      title: t => t('financial.title'),
      subtitle: t => t(`sarlaft.${sarlaftPersonType}`),
      subtitlePlacement: 'top',
      fields: [
        {
          component: TextFieldFilled,
          fieldProps: {
            parse: fieldParser.onlyNumbers
          },
          muiInputProps: {
            InputProps: {
              inputComponent: MoneyFormat,
              inputProps: {
                ...InputModeProps.numeric,
                maxLength: MAX_MONEYFIELD_LENGTH
              }
            }
          },
          name: Work.MonthlyIncome,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Work.MonthlyIncome}.label`)
          },
          validate:
            fieldIsRequired(Work.MonthlyIncome, optionalFieldNames) &&
            validateWith(validators.required)
        },
        {
          component: TextFieldFilled,
          fieldProps: {
            parse: fieldParser.onlyNumbers
          },
          muiInputProps: {
            InputProps: {
              inputComponent: MoneyFormat,
              inputProps: {
                ...InputModeProps.numeric,
                maxLength: MAX_MONEYFIELD_LENGTH
              }
            }
          },
          name: Work.MonthlyDischarge,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Work.MonthlyDischarge}.label`)
          },
          validate:
            fieldIsRequired(Work.MonthlyDischarge, optionalFieldNames) &&
            validateWith(validators.required)
        },
        {
          component: TextFieldFilled,
          fieldProps: {
            parse: fieldParser.onlyNumbers
          },
          muiInputProps: {
            InputProps: {
              inputComponent: MoneyFormat,
              inputProps: {
                ...assetsFieldConfig.muiInputProps.InputProps.inputProps,
                ...InputModeProps.numeric,
                maxLength: MAX_MONEYFIELD_LENGTH
              }
            }
          },
          name: Work.Assets,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Work.Assets}.label`),
            helperText: t => t(`${Work.Assets}.helperText`)
          },
          validate: assetsFieldConfig.validate
        },
        {
          component: TextFieldFilled,
          fieldProps: {
            parse: fieldParser.onlyNumbers
          },
          muiInputProps: {
            InputProps: {
              inputComponent: MoneyFormat,
              inputProps: {
                ...InputModeProps.numeric,
                maxLength: MAX_MONEYFIELD_LENGTH
              }
            }
          },
          name: Work.Liabilities,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Work.Liabilities}.label`),
            helperText: t => t(`${FieldNames.Insured.Work.Liabilities}.helperText`)
          },
          validate:
            fieldIsRequired(Work.Liabilities, optionalFieldNames) &&
            validateWith(validators.required)
        },
        {
          component: TextFieldFilled,
          muiInputProps: {
            InputProps: {
              inputComponent: MoneyFormat,
              disabled: true
            }
          },
          name: FieldNames[sarlaftPersonType].Work.Wealth,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Work.Wealth}.label`)
          }
        }
      ]
    },
    {
      section: sarlaftPersonType === SarlaftPersonType.Insured ? insuredSection : payerSection,
      id: StepIds[`${sarlaftPersonType}SourceOfIncome`],
      next: StepIds[`${sarlaftPersonType}PublicResources`],
      title: t => t('sourceOfIncome.title'),
      subtitle: t => t(`sarlaft.${sarlaftPersonType}`),
      subtitlePlacement: 'top',
      fields: [getPrincipalSourceOfIncomeField(sarlaftPersonType, undefined, optionalFieldNames)]
    }
  ];
};

export { financialSteps };
