import {
  AutoCompleteTextFieldFilled,
  CardButtonGroup,
  TextFieldFilled
} from '@comparaonline/ui-form-fields-next';
import { Step } from '@comparaonline/ui-wizard-step-manager';
import { validateWith } from '@comparaonline/ui-forms-validators/dist/validateWith';
import { InputModeProps } from '@comparaonline/ui-input-helpers';

import { MoneyFormat } from '../components';
import {
  FieldNames,
  StepIds,
  StepNames,
  booleanOptions,
  currencyOperationOptions,
  foreignProductTypeOptions
} from '../constants';
import { SarlaftPersonType } from '../types';
import { fieldIsRequired, fieldParser, fieldValueIsTrue } from '../helpers';
import * as validators from '../validators';
import { SectionProps } from '@comparaonline/ui-wizard-step-manager/dist/types';

const foreignTaxationSection: SectionProps = {
  id: 'foreignTaxation',
  name: () => StepNames.ForeignTaxation
};

const hasForeignCurrencyOperations = (sarlaftPersonType: SarlaftPersonType) =>
  fieldValueIsTrue(FieldNames[sarlaftPersonType].ForeignTaxation.HasForeignCurrencyOperations);

const hasFinancialProducts = (sarlaftPersonType: SarlaftPersonType) =>
  fieldValueIsTrue(FieldNames[sarlaftPersonType].ForeignTaxation.HasFinancialProducts);

export const foreignTaxation = (
  sarlaftPersonType: SarlaftPersonType,
  optionalFieldNames?: string[]
): Step[] => [
  {
    section: foreignTaxationSection,
    id: StepIds[`${sarlaftPersonType}ForeignTaxation`],
    next: StepIds[`${sarlaftPersonType}Insurance`],
    title: t => t('foreignTaxation.title'),
    subtitle: t => t(`sarlaft.${sarlaftPersonType}`),
    subtitlePlacement: 'top',
    fields: [
      {
        component: CardButtonGroup,
        labelPosition: 'center',
        name: FieldNames[sarlaftPersonType].ForeignTaxation.HasForeignCurrencyOperations,
        options: booleanOptions,
        showLabel: true,
        texts: {
          label: t => t(`${FieldNames.Insured.ForeignTaxation.HasForeignCurrencyOperations}.label`),
          labelTooltip: t =>
            t(`${FieldNames.Insured.ForeignTaxation.HasForeignCurrencyOperations}.labelTooltip`)
        },
        validate:
          fieldIsRequired(
            FieldNames[sarlaftPersonType].ForeignTaxation.HasForeignCurrencyOperations,
            optionalFieldNames
          ) && validateWith(validators.requiredWithFalsies)
      },
      {
        component: TextFieldFilled,
        conditionToRender: hasForeignCurrencyOperations(sarlaftPersonType),
        name: FieldNames[sarlaftPersonType].ForeignTaxation.CurrencyOperations,
        options: currencyOperationOptions,
        select: true,
        showLabel: true,
        texts: {
          label: t => t(`${FieldNames.Insured.ForeignTaxation.CurrencyOperations}.label`)
        },
        validate:
          fieldIsRequired(
            FieldNames[sarlaftPersonType].ForeignTaxation.CurrencyOperations,
            optionalFieldNames
          ) && validateWith(validators.requiredWithFalsies)
      },
      {
        component: TextFieldFilled,
        conditionToRender: hasForeignCurrencyOperations(sarlaftPersonType),
        muiInputProps: {
          InputProps: {
            inputComponent: MoneyFormat,
            inputProps: {
              ...InputModeProps.numeric
            }
          }
        },
        name: FieldNames[sarlaftPersonType].ForeignTaxation.AmountCurrencyOperations,
        showLabel: true,
        texts: {
          label: t => t(`${FieldNames.Insured.ForeignTaxation.AmountCurrencyOperations}.label`)
        },
        validate:
          fieldIsRequired(
            FieldNames[sarlaftPersonType].ForeignTaxation.AmountCurrencyOperations,
            optionalFieldNames
          ) && validateWith(validators.required)
      },
      {
        component: CardButtonGroup,
        labelPosition: 'center',
        name: FieldNames[sarlaftPersonType].ForeignTaxation.HasFinancialProducts,
        options: booleanOptions,
        showLabel: true,
        texts: {
          label: t => t(`${FieldNames.Insured.ForeignTaxation.HasFinancialProducts}.label`),
          labelTooltip: t =>
            t(`${FieldNames.Insured.ForeignTaxation.HasFinancialProducts}.labelTooltip`)
        },
        validate:
          fieldIsRequired(
            FieldNames[sarlaftPersonType].ForeignTaxation.HasFinancialProducts,
            optionalFieldNames
          ) && validateWith(validators.requiredWithFalsies)
      },
      {
        component: TextFieldFilled,
        conditionToRender: hasFinancialProducts(sarlaftPersonType),
        name: FieldNames[sarlaftPersonType].ForeignTaxation.TypeOfProduct,
        options: foreignProductTypeOptions,
        select: true,
        showLabel: true,
        texts: {
          label: t => t(`${FieldNames.Insured.ForeignTaxation.TypeOfProduct}.label`)
        },
        validate:
          fieldIsRequired(
            FieldNames[sarlaftPersonType].ForeignTaxation.TypeOfProduct,
            optionalFieldNames
          ) && validateWith(validators.requiredWithFalsies)
      },
      {
        component: TextFieldFilled,
        conditionToRender: hasFinancialProducts(sarlaftPersonType),
        name: FieldNames[sarlaftPersonType].ForeignTaxation.NumberOfProduct,
        showLabel: true,
        texts: {
          helperText: t => t(`${FieldNames.Insured.ForeignTaxation.NumberOfProduct}.helperText`),
          label: t => t(`${FieldNames.Insured.ForeignTaxation.NumberOfProduct}.label`)
        },
        validate:
          fieldIsRequired(
            FieldNames[sarlaftPersonType].ForeignTaxation.NumberOfProduct,
            optionalFieldNames
          ) && validateWith(validators.required)
      },
      {
        component: TextFieldFilled,
        conditionToRender: hasFinancialProducts(sarlaftPersonType),
        name: FieldNames[sarlaftPersonType].ForeignTaxation.FinancialProductAmount,
        showLabel: true,
        fieldProps: {
          parse: fieldParser.onlyNumbers
        },
        muiInputProps: {
          adornment: {
            position: 'start'
          },
          InputProps: {
            inputProps: {
              ...InputModeProps.numeric
            }
          }
        },
        texts: {
          adornment: t =>
            t(`${FieldNames.Insured.ForeignTaxation.FinancialProductAmount}.adornment`),
          label: t => t(`${FieldNames.Insured.ForeignTaxation.FinancialProductAmount}.label`),
          helperText: t =>
            t(`${FieldNames.Insured.ForeignTaxation.FinancialProductAmount}.helperText`)
        },
        validate:
          fieldIsRequired(
            FieldNames[sarlaftPersonType].ForeignTaxation.FinancialProductAmount,
            optionalFieldNames
          ) && validateWith(validators.required)
      },
      {
        component: TextFieldFilled,
        conditionToRender: hasFinancialProducts(sarlaftPersonType),
        name: FieldNames[sarlaftPersonType].ForeignTaxation.FinancialProductEntity,
        showLabel: true,
        texts: {
          label: t => t(`${FieldNames.Insured.ForeignTaxation.FinancialProductEntity}.label`)
        },
        validate:
          fieldIsRequired(
            FieldNames[sarlaftPersonType].ForeignTaxation.FinancialProductEntity,
            optionalFieldNames
          ) && validateWith(validators.required)
      },
      {
        checkForDataOptions: true,
        component: AutoCompleteTextFieldFilled,
        conditionToRender: hasFinancialProducts(sarlaftPersonType),
        name: FieldNames[sarlaftPersonType].ForeignTaxation.FinancialProductCountry,
        select: true,
        showLabel: true,
        texts: {
          label: t => t(`${FieldNames.Insured.ForeignTaxation.FinancialProductCountry}.label`)
        },
        validate:
          fieldIsRequired(
            FieldNames[sarlaftPersonType].ForeignTaxation.FinancialProductCountry,
            optionalFieldNames
          ) && validateWith(validators.required)
      },
      {
        component: TextFieldFilled,
        conditionToRender: hasFinancialProducts(sarlaftPersonType),
        fieldProps: {
          parse: fieldParser.noNumbers
        },
        name: FieldNames[sarlaftPersonType].ForeignTaxation.FinancialProductCity,
        showLabel: true,
        texts: {
          label: t => t(`${FieldNames.Insured.ForeignTaxation.FinancialProductCity}.label`)
        },
        validate:
          fieldIsRequired(
            FieldNames[sarlaftPersonType].ForeignTaxation.FinancialProductCity,
            optionalFieldNames
          ) && validateWith(validators.required)
      }
    ]
  }
];
