import { CardButtonGroup, TextFieldFilled } from '@comparaonline/ui-form-fields-next';
import { Step } from '@comparaonline/ui-wizard-step-manager';
import { validateWith } from '@comparaonline/ui-forms-validators/dist/validateWith';
import { InputModeProps } from '@comparaonline/ui-input-helpers';

import { MoneyFormat } from '../components';
import {
  FieldNames,
  MAX_YEAR_DIGITS,
  StepIds,
  StepNames,
  booleanOptions,
  insuranceCategoryOptions,
  insuranceResultOptions
} from '../constants';
import { SarlaftPersonType } from '../types';
import { fieldIsRequired, fieldParser, fieldValueIsTrue } from '../helpers';
import * as validators from '../validators';
import { SectionProps } from '@comparaonline/ui-wizard-step-manager/dist/types';
import { bypassValidator } from '../validators';

export const insuranceSteps = (
  sarlaftPersonType: SarlaftPersonType,
  next: string,
  optionalFieldNames?: string[]
): Step[] => {
  const insuranceFieldName = FieldNames[sarlaftPersonType].Insurance;
  const section: SectionProps = {
    id: `${sarlaftPersonType}-insurance`,
    name: () => StepNames.Insurance
  };

  const currentYear = new Date().getFullYear();

  return [
    {
      section,
      id: StepIds[`${sarlaftPersonType}Insurance`],
      next,
      title: t => t('insurance.title'),
      subtitle: t => t(`sarlaft.${sarlaftPersonType}`),
      subtitlePlacement: 'top',
      fields: [
        {
          component: CardButtonGroup,
          name: FieldNames[sarlaftPersonType].HasClaims,
          labelPosition: 'center',
          options: booleanOptions,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.HasClaims}.label`),
            labelTooltip: t => t(`${FieldNames.Insured.HasClaims}.labelTooltip`)
          },
          validate:
            fieldIsRequired(FieldNames[sarlaftPersonType].HasClaims, optionalFieldNames) &&
            validateWith(validators.requiredWithFalsies)
        },
        {
          conditionToRender: fieldValueIsTrue(FieldNames[sarlaftPersonType].HasClaims),
          component: TextFieldFilled,
          fieldProps: {
            parse: fieldParser.onlyNumbers
          },
          muiInputProps: {
            InputProps: {
              inputProps: {
                ...InputModeProps.numeric,
                maxLength: MAX_YEAR_DIGITS
              }
            }
          },
          name: insuranceFieldName.Year,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Insurance.Year}.label`)
          },
          validate: validateWith(
            fieldIsRequired(FieldNames[sarlaftPersonType].HasClaims, optionalFieldNames)
              ? validators.required
              : bypassValidator,
            validators.maxNumber({ max: currentYear, variables: { max: currentYear } })
          )
        },
        {
          conditionToRender: fieldValueIsTrue(FieldNames[sarlaftPersonType].HasClaims),
          component: TextFieldFilled,
          name: insuranceFieldName.Category,
          options: insuranceCategoryOptions,
          select: true,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Insurance.Category}.label`)
          },
          validate:
            fieldIsRequired(insuranceFieldName.Category, optionalFieldNames) &&
            validateWith(validators.requiredWithFalsies)
        },
        {
          conditionToRender: fieldValueIsTrue(FieldNames[sarlaftPersonType].HasClaims),
          component: TextFieldFilled,
          name: insuranceFieldName.Company,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Insurance.Company}.label`)
          },
          validate:
            fieldIsRequired(insuranceFieldName.Company, optionalFieldNames) &&
            validateWith(validators.required)
        },
        {
          conditionToRender: fieldValueIsTrue(FieldNames[sarlaftPersonType].HasClaims),
          component: TextFieldFilled,
          fieldProps: {
            parse: fieldParser.onlyNumbers
          },
          muiInputProps: {
            InputProps: {
              inputComponent: MoneyFormat,
              inputProps: {
                ...InputModeProps.numeric
              }
            }
          },
          name: insuranceFieldName.Amount,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Insurance.Amount}.label`)
          },
          validate:
            fieldIsRequired(insuranceFieldName.Amount, optionalFieldNames) &&
            validateWith(validators.required)
        },
        {
          conditionToRender: fieldValueIsTrue(FieldNames[sarlaftPersonType].HasClaims),
          component: TextFieldFilled,
          name: insuranceFieldName.Result,
          options: insuranceResultOptions,
          select: true,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Insurance.Result}.label`)
          },
          validate:
            fieldIsRequired(insuranceFieldName.Result, optionalFieldNames) &&
            validateWith(validators.requiredWithFalsies)
        }
      ]
    }
  ];
};
