import { AutoCompleteTextFieldFilled, TextFieldFilled } from '@comparaonline/ui-form-fields-next';
import { Step } from '@comparaonline/ui-wizard-step-manager';
import { validateWith } from '@comparaonline/ui-forms-validators/dist/validateWith';
import { InputModeProps } from '@comparaonline/ui-input-helpers';

import {
  FieldNames,
  MAX_CHILDREN_QUANTIY_LENGTH,
  StepIds,
  civilStatusOptions,
  educationLevelOptions,
  housingTypeOptions,
  insuredSection,
  payerSection,
  socioeconomicLevelOptions
} from '../constants';
import { fieldIsRequired, fieldParser } from '../helpers';
import { SarlaftPersonType } from '../types';
import * as validators from '../validators';

import { getAddressField } from './fields/address';
import { getBirthdateField } from './fields/birthdate';
import { getCityField } from './fields/city';
import { getDateIssuedField } from './fields/dateIssued';
import { getEmailField } from './fields/email';
import { getFirstNameField } from './fields/firstName';
import { getGenderField } from './fields/gender';
import { getIdentificationNumberField } from './fields/identificationNumber';
import { getIdentificationTypeField } from './fields/identificationType';
import { getLandlineField } from './fields/landline';
import { getLastNameField } from './fields/lastName';
import { getMiddleNameField } from './fields/middleName';
import { getNeighborhoodField } from './fields/neigborhood';
import { getPlaceIssuedField } from './fields/placeIssued';
import { getPhoneField } from './fields/phone';
import { getSecondLastNameField } from './fields/secondLastName';

export const insuredSteps = (
  sarlaftPersonType: SarlaftPersonType,
  optionalFieldNames?: string[]
): Step[] => {
  const personFieldName = FieldNames[sarlaftPersonType].Person;
  const section = sarlaftPersonType === SarlaftPersonType.Insured ? insuredSection : payerSection;
  const subtitle = t => t(`sarlaft.${sarlaftPersonType}`);
  const title = t => t('insured.title');

  return [
    {
      id: StepIds[`${sarlaftPersonType}`],
      next: StepIds[`${sarlaftPersonType}Identification`],
      section,
      subtitlePlacement: 'top',
      subtitle,
      title,
      fields: [
        getFirstNameField(sarlaftPersonType, optionalFieldNames),
        getMiddleNameField(sarlaftPersonType),
        getLastNameField(sarlaftPersonType, optionalFieldNames),
        getSecondLastNameField(sarlaftPersonType),
        getGenderField(sarlaftPersonType, optionalFieldNames),
        getBirthdateField(sarlaftPersonType, optionalFieldNames),
        {
          checkForDataOptions: true,
          component: AutoCompleteTextFieldFilled,
          name: personFieldName.Nationality,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Person.Nationality}.label`),
            placeholder: t => t(`${FieldNames.Insured.Person.Nationality}.placeholder`)
          },
          validate:
            fieldIsRequired(personFieldName.Nationality, optionalFieldNames) &&
            validateWith(validators.required)
        },
        {
          checkForDataOptions: true,
          component: AutoCompleteTextFieldFilled,
          name: personFieldName.Birthplace,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Person.Birthplace}.label`),
            placeholder: t => t(`${FieldNames.Insured.Person.Birthplace}.placeholder`)
          },
          validate:
            fieldIsRequired(personFieldName.Birthplace, optionalFieldNames) &&
            validateWith(validators.required)
        }
      ]
    },
    {
      id: StepIds[`${sarlaftPersonType}Identification`],
      next: StepIds[`${sarlaftPersonType}Address`],
      section,
      subtitlePlacement: 'top',
      subtitle,
      title,
      fields: [
        [
          getIdentificationTypeField(sarlaftPersonType, optionalFieldNames),
          getIdentificationNumberField(sarlaftPersonType, optionalFieldNames)
        ],
        getPlaceIssuedField(sarlaftPersonType, optionalFieldNames),
        getDateIssuedField(sarlaftPersonType, optionalFieldNames),
        {
          component: TextFieldFilled,
          name: personFieldName.CivilStatus,
          options: civilStatusOptions,
          select: true,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Person.CivilStatus}.label`),
            placeholder: t => t(`${FieldNames.Insured.Person.CivilStatus}.placeholder`)
          },
          validate:
            fieldIsRequired(personFieldName.CivilStatus, optionalFieldNames) &&
            validateWith(validators.required)
        },
        {
          component: TextFieldFilled,
          name: personFieldName.EducationLevel,
          options: educationLevelOptions,
          select: true,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Person.EducationLevel}.label`),
            placeholder: t => t(`${FieldNames.Insured.Person.EducationLevel}.placeholder`)
          },
          validate:
            fieldIsRequired(personFieldName.EducationLevel, optionalFieldNames) &&
            validateWith(validators.required)
        },
        {
          component: TextFieldFilled,
          fieldProps: {
            parse: fieldParser.onlyNumbers
          },
          muiInputProps: {
            InputProps: {
              inputProps: {
                ...InputModeProps.numeric,
                maxLength: MAX_CHILDREN_QUANTIY_LENGTH
              }
            }
          },
          name: personFieldName.ChildrenQuantity,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Person.ChildrenQuantity}.label`),
            placeholder: t => t(`${FieldNames.Insured.Person.ChildrenQuantity}.placeholder`)
          },
          validate:
            fieldIsRequired(personFieldName.ChildrenQuantity, optionalFieldNames) &&
            validateWith(validators.required)
        }
      ]
    },
    {
      id: StepIds[`${sarlaftPersonType}Address`],
      next: StepIds[`${sarlaftPersonType}Contact`],
      section,
      subtitlePlacement: 'top',
      subtitle,
      title,
      fields: [
        getAddressField(sarlaftPersonType, optionalFieldNames),
        getNeighborhoodField(sarlaftPersonType, optionalFieldNames),
        getCityField(sarlaftPersonType, optionalFieldNames),
        {
          component: TextFieldFilled,
          name: personFieldName.SocioeconomicLevel,
          options: socioeconomicLevelOptions,
          select: true,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Person.SocioeconomicLevel}.label`),
            placeholder: t => t(`${FieldNames.Insured.Person.SocioeconomicLevel}.placeholder`)
          },
          validate:
            fieldIsRequired(personFieldName.SocioeconomicLevel, optionalFieldNames) &&
            validateWith(validators.requiredWithFalsies)
        },
        {
          component: TextFieldFilled,
          name: personFieldName.HousingType,
          options: housingTypeOptions,
          select: true,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Person.HousingType}.label`),
            placeholder: t => t(`${FieldNames.Insured.Person.HousingType}.placeholder`)
          },
          validate:
            fieldIsRequired(personFieldName.HousingType, optionalFieldNames) &&
            validateWith(validators.required)
        }
      ]
    },
    {
      id: StepIds[`${sarlaftPersonType}Contact`],
      next: StepIds[`${sarlaftPersonType}OccupationType`],
      section,
      subtitlePlacement: 'top',
      subtitle,
      title,
      fields: [
        getLandlineField(sarlaftPersonType, optionalFieldNames),
        getPhoneField(sarlaftPersonType, optionalFieldNames),
        getEmailField(sarlaftPersonType, optionalFieldNames)
      ]
    }
  ];
};
