import { FormApi } from 'final-form';
import { AutoCompleteTextFieldFilled, TextFieldFilled } from '@comparaonline/ui-form-fields-next';
import { Step } from '@comparaonline/ui-wizard-step-manager';
import { validateWith } from '@comparaonline/ui-forms-validators/dist/validateWith';
import { ColombiaPhoneInputMask } from '@comparaonline/ui-text-masks';
import { OccupationTypes } from '@comparaonline/common-schema';

import {
  FieldNames,
  MAX_TEXTFIELD_LENGTH,
  StepIds,
  ciiuCodeOptions,
  companyTypeOptions,
  insuredSection,
  occupationTypeOptions,
  payerSection
} from '../constants';
import { fieldIsRequired, getFieldStateValue, isUnemployed } from '../helpers';
import { SarlaftPersonType } from '../types';
import * as validators from '../validators';

const occupationTypeIs = (sarlaftPersonType: SarlaftPersonType, compareTo: string) => (
  formApi: FormApi
): boolean =>
  getFieldStateValue(formApi, FieldNames[sarlaftPersonType].Work.OccupationType) === compareTo;

const isEmployed = (sarlaftPersonType: SarlaftPersonType) => (formApi: FormApi): boolean =>
  !isUnemployed(getFieldStateValue(formApi, FieldNames[sarlaftPersonType].Work.OccupationType));

const isHiredOrRetired = (sarlaftPersonType: SarlaftPersonType) => (formApi: FormApi): boolean =>
  occupationTypeIs(sarlaftPersonType, OccupationTypes.Hired)(formApi) ||
  occupationTypeIs(sarlaftPersonType, OccupationTypes.Retired)(formApi);

const occupationSteps = ({
  sarlaftPersonType,
  nextStep,
  optionalFieldNames
}: {
  sarlaftPersonType: SarlaftPersonType;
  nextStep?: string;
  optionalFieldNames?: string[];
}): Step[] => {
  const workFieldName = FieldNames[sarlaftPersonType].Work;

  return [
    {
      section: sarlaftPersonType === SarlaftPersonType.Insured ? insuredSection : payerSection,
      id: StepIds[`${sarlaftPersonType}OccupationType`],
      next: nextStep || StepIds[`${sarlaftPersonType}Financial`],
      title: t => t('occupation.title'),
      subtitle: t => t(`sarlaft.${sarlaftPersonType}`),
      subtitlePlacement: 'top',
      fields: [
        {
          component: TextFieldFilled,
          name: workFieldName.OccupationType,
          options: occupationTypeOptions,
          select: true,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Work.OccupationType}.label`),
            placeholder: t => t(`${FieldNames.Insured.Work.OccupationType}.placeholder`)
          },
          validate:
            fieldIsRequired(workFieldName.OccupationType, optionalFieldNames) &&
            validateWith(validators.requiredWithFalsies)
        },
        {
          component: AutoCompleteTextFieldFilled,
          conditionToRender: occupationTypeIs(sarlaftPersonType, OccupationTypes.Freelance),
          name: workFieldName.CiiuCode,
          options: ciiuCodeOptions,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Work.CiiuCode}.label`),
            placeholder: t => t(`${FieldNames.Insured.Work.CiiuCode}.placeholder`)
          },
          validate:
            fieldIsRequired(workFieldName.CiiuCode, optionalFieldNames) &&
            validateWith(validators.requiredWithFalsies)
        },
        {
          conditionToRender: isHiredOrRetired(sarlaftPersonType),
          component: TextFieldFilled,
          muiInputProps: {
            InputProps: {
              inputProps: {
                maxLength: MAX_TEXTFIELD_LENGTH
              }
            }
          },
          name: workFieldName.Company,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Work.Company}.label`)
          },
          validate:
            fieldIsRequired(workFieldName.Company, optionalFieldNames) &&
            validateWith(validators.required)
        },
        {
          conditionToRender: isEmployed(sarlaftPersonType),
          component: TextFieldFilled,
          muiInputProps: {
            InputProps: {
              inputProps: {
                maxLength: MAX_TEXTFIELD_LENGTH
              }
            }
          },
          name: workFieldName.Occupation,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Work.Occupation}.label`)
          },
          validate:
            fieldIsRequired(workFieldName.Occupation, optionalFieldNames) &&
            validateWith(validators.required)
        },
        {
          component: TextFieldFilled,
          conditionToRender: isHiredOrRetired(sarlaftPersonType),
          muiInputProps: {
            InputProps: {
              inputProps: {
                maxLength: MAX_TEXTFIELD_LENGTH
              }
            }
          },
          name: workFieldName.Position,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Work.Position}.label`)
          },
          validate:
            fieldIsRequired(workFieldName.Position, optionalFieldNames) &&
            validateWith(validators.required)
        },
        {
          checkForDataOptions: true,
          conditionToRender: isEmployed(sarlaftPersonType),
          component: AutoCompleteTextFieldFilled,
          name: workFieldName.CompanyCity,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Work.CompanyCity}.label`),
            placeholder: t => t(`${FieldNames.Insured.Work.CompanyCity}.placeholder`)
          },
          validate:
            fieldIsRequired(workFieldName.CompanyCity, optionalFieldNames) &&
            validateWith(validators.required)
        },
        {
          conditionToRender: isEmployed(sarlaftPersonType),
          component: TextFieldFilled,
          name: workFieldName.CompanyAddress,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Work.CompanyAddress}.label`)
          },
          validate:
            fieldIsRequired(workFieldName.CompanyAddress, optionalFieldNames) &&
            validateWith(validators.required)
        },
        {
          conditionToRender: isEmployed(sarlaftPersonType),
          component: TextFieldFilled,
          muiInputProps: {
            InputProps: {
              inputComponent: ColombiaPhoneInputMask,
              inputProps: {
                type: 'tel'
              }
            }
          },
          name: workFieldName.CompanyPhone,
          showLabel: true,
          texts: {
            helperText: t => t(`${FieldNames.Insured.Work.CompanyPhone}.helperText`),
            label: t => t(`${FieldNames.Insured.Work.CompanyPhone}.label`),
            placeholder: t => t(`${FieldNames.Insured.Work.CompanyPhone}.placeholder`)
          },
          validate: validateWith(validators.validatePhone({ landLine: true, mobile: true }))
        },
        {
          conditionToRender: isHiredOrRetired(sarlaftPersonType),
          component: TextFieldFilled,
          name: workFieldName.CompanyType,
          options: companyTypeOptions,
          select: true,
          showLabel: true,
          texts: {
            label: t => t(`${FieldNames.Insured.Work.CompanyType}.label`),
            placeholder: t => t(`${FieldNames.Insured.Work.CompanyType}.placeholder`)
          },
          validate:
            fieldIsRequired(workFieldName.CompanyType, optionalFieldNames) &&
            validateWith(validators.requiredWithFalsies)
        }
      ]
    }
  ];
};

export { occupationSteps };
