import { FormApi, getIn } from 'final-form';
import { CicoPaymentMethod } from '@comparaonline/cico-funnel-schema';
import { FormatMessage } from '@comparaonline/react-intl-hooks';
import {
  AlertSeverity,
  AlertWarning,
  AutoCompleteTextFieldFilled as AutoCompleteTextField,
  CardButtonGroup,
  CardButtonVariant,
  RequiredFileHandlers,
  TextFieldFilled as TextField
} from '@comparaonline/ui-form-fields-next';
import { validateWith } from '@comparaonline/ui-forms-validators/dist/validateWith';
import { InputModeProps } from '@comparaonline/ui-input-helpers';
import { CreditCardExpeditionMask } from '@comparaonline/ui-text-masks';
import { IFieldProps as FieldProps, Step } from '@comparaonline/ui-wizard-step-manager';

import { Subsidiaries } from '../components';
import {
  creditCardInstallmentsOptions,
  FieldNames,
  MAX_CREDIT_CARD_IDENTIFICAION_NUMBER_LENGTH,
  MAX_CREDIT_CARD_NAME_LENGTH,
  MAX_CREDIT_CARD_NUMBER_LENGTH,
  MAX_PSE_IDENTIFICATION_NUMBER_LENGTH,
  MIN_CREDIT_CARD_IDENTIFICAION_NUMBER_LENGTH,
  PaymentSectionId,
  PseIdentificationTypeOptions,
  PsePersonalTypeOptions,
  StepIds,
  StepNames
} from '../constants';

import {
  creditCard,
  creditCardExpedition,
  creditCardType,
  email,
  invalidCardExpeditionFormat,
  length,
  required,
  requiredWithFalsies
} from '../validators';
import { AllowedPaymentMethod, Offer } from '../types';
import { fieldParser, getPaymentMethodOptions, isPaymentMethod } from '../helpers';

const PaymentMethodFieldName = FieldNames.Payment.Method;
const displayForCreditCard = (formApi: FormApi): boolean =>
  isPaymentMethod(formApi, CicoPaymentMethod.CreditCard);

const creditCardFields = (): (FieldProps | FieldProps[])[] => [
  {
    conditionToRender: displayForCreditCard,
    component: TextField,
    name: FieldNames.Payment.CreditCard.Name,
    muiInputProps: {
      InputProps: {
        inputProps: {
          maxLength: MAX_CREDIT_CARD_NAME_LENGTH
        }
      }
    },
    showLabel: true,
    texts: {
      label: (t: FormatMessage): string => t(`${FieldNames.Payment.CreditCard.Name}.label`)
    },
    validate: validateWith(required)
  },
  {
    conditionToRender: displayForCreditCard,
    component: TextField,
    name: FieldNames.Payment.CreditCard.IdentificationNumber,
    fieldProps: {
      parse: fieldParser.thousand
    },
    muiInputProps: {
      InputProps: {
        inputProps: {
          maxLength: MAX_CREDIT_CARD_IDENTIFICAION_NUMBER_LENGTH
        }
      }
    },
    showLabel: true,
    texts: {
      helperText: (t: FormatMessage): string =>
        t(`${FieldNames.Payment.CreditCard.IdentificationNumber}.helperText`),
      label: (t: FormatMessage): string =>
        t(`${FieldNames.Payment.CreditCard.IdentificationNumber}.label`)
    },
    validate: validateWith(
      required,
      length({
        cleanValue: value => value.replace(/\D/g, ''),
        min: MIN_CREDIT_CARD_IDENTIFICAION_NUMBER_LENGTH
      })
    )
  },
  {
    conditionToRender: displayForCreditCard,
    component: TextField,
    name: FieldNames.Payment.CreditCard.Email,
    showLabel: true,
    texts: {
      label: (t: FormatMessage): string => t(`${FieldNames.Payment.CreditCard.Email}.label`)
    },
    validate: validateWith(required, email)
  },
  {
    conditionToRender: displayForCreditCard,
    component: TextField,
    name: FieldNames.Payment.CreditCard.CardNumber,
    fieldProps: {
      parse: fieldParser.creditCard
    },
    muiInputProps: {
      InputProps: {
        inputProps: {
          ...InputModeProps.numeric,
          maxLength: MAX_CREDIT_CARD_NUMBER_LENGTH
        }
      }
    },
    showLabel: true,
    texts: {
      label: (t: FormatMessage): string => t(`${FieldNames.Payment.CreditCard.CardNumber}.label`)
    },
    validate: validateWith(required, creditCard, creditCardType)
  },
  [
    {
      conditionToRender: displayForCreditCard,
      component: TextField,
      name: FieldNames.Payment.CreditCard.CardExpedition,
      fieldProps: {
        parse: fieldParser.onlyNumbers
      },
      muiInputProps: {
        InputProps: {
          inputComponent: CreditCardExpeditionMask,
          inputProps: {
            ...InputModeProps.numeric
          }
        }
      },
      showLabel: true,
      texts: {
        helperText: (t: FormatMessage): string =>
          t(`${FieldNames.Payment.CreditCard.CardExpedition}.helperText`),
        label: (t: FormatMessage): string =>
          t(`${FieldNames.Payment.CreditCard.CardExpedition}.label`),
        placeholder: (t: FormatMessage): string =>
          t(`${FieldNames.Payment.CreditCard.CardExpedition}.placeholder`)
      },
      validate: validateWith(required, invalidCardExpeditionFormat, creditCardExpedition)
    },
    {
      conditionToRender: displayForCreditCard,
      component: TextField,
      name: FieldNames.Payment.CreditCard.Installments,
      options: creditCardInstallmentsOptions,
      select: true,
      showLabel: true,
      texts: {
        label: (t: FormatMessage): string =>
          t(`${FieldNames.Payment.CreditCard.Installments}.label`)
      },
      validate: validateWith(required)
    }
  ]
];

const PSEFields: FieldProps[] = [
  {
    checkForDataOptions: true,
    component: AutoCompleteTextField,
    name: FieldNames.Payment.PSE.Bank,
    validate: validateWith(requiredWithFalsies)
  },
  {
    component: TextField,
    name: FieldNames.Payment.PSE.Name,
    validate: validateWith(required)
  },
  {
    component: TextField,
    name: FieldNames.Payment.PSE.PersonalType,
    options: PsePersonalTypeOptions,
    select: true,
    validate: validateWith(requiredWithFalsies)
  },
  {
    component: AutoCompleteTextField,
    name: FieldNames.Payment.PSE.IdentificationType,
    options: PseIdentificationTypeOptions,
    select: true,
    validate: validateWith(requiredWithFalsies)
  },
  {
    component: TextField,
    muiInputProps: {
      InputProps: {
        inputProps: {
          ...InputModeProps.numeric,
          maxLength: MAX_PSE_IDENTIFICATION_NUMBER_LENGTH
        }
      }
    },
    name: FieldNames.Payment.PSE.IdentificationNumber,
    validate: validateWith(required)
  }
].map(field => ({
  ...field,
  conditionToRender: (formApi: FormApi) => isPaymentMethod(formApi, CicoPaymentMethod.Pse),
  showLabel: true,
  texts: {
    label: (t: FormatMessage) => t(`${field.name}.label`)
  }
}));

type PaymentStep = (input: {
  bffURL: string;
  fileHandler: RequiredFileHandlers;
  funnelId: string;
  offer: Offer;
  ownerViewing: boolean;
  paymentMethods: AllowedPaymentMethod[];
}) => Step[];

export const paymentStep: PaymentStep = ({ paymentMethods, offer }) => [
  {
    section: {
      id: PaymentSectionId,
      name: () => StepNames.Payment
    },
    id: StepIds.Payment,
    submitText: t => t('payment.btn'),
    title: t => t(`${PaymentSectionId}.title`),
    fields: [
      {
        component: CardButtonGroup,
        name: PaymentMethodFieldName,
        options: getPaymentMethodOptions(paymentMethods, offer),
        validate: validateWith(requiredWithFalsies),
        variant: CardButtonVariant.Horizontal,
        warning: {
          component: AlertWarning,
          severity: AlertSeverity.Info,
          text: (t, values) => {
            const i18nPrefix = `${PaymentMethodFieldName}.warning`;
            const selectedPaymentMethod = getIn(values, PaymentMethodFieldName);
            const insuranceCompany =
              offer?.product?.division?.company?.name ||
              t(`${i18nPrefix}.fallback.insuranceCompany`);
            return t(`${i18nPrefix}.${selectedPaymentMethod}`, { insuranceCompany });
          }
        }
      },
      {
        component: Subsidiaries,
        conditionToRender: (formApi: FormApi) => isPaymentMethod(formApi, CicoPaymentMethod.Cash),
        name: 'subsidaries'
      },
      ...creditCardFields(),
      ...PSEFields
    ]
  }
];
