import { CicoSarlaftType } from '@comparaonline/cico-funnel-schema';
import { IFieldProps } from '@comparaonline/ui-wizard-step-manager';
import { getCarSteps as getCarStepsConfig } from './car';
import { getDigitalSignatureSteps } from './digitalSignature';
import { financialSteps } from './financialSteps';
import { foreignTaxation } from './foreignTaxation';
import { getFileSteps } from './fileStep';
import { insuranceSteps } from './insurance';
import { insuredSteps } from './insuredSteps';
import { occupationSteps } from './occupationSteps';
import { paymentStep } from './payment';
import { publicResourcesSteps } from './publicResourcesSteps';

import { StepIds } from '../constants';
import { isTLCPlate, getInspectionType } from '../helpers';
import {
  AllowedInspectionType,
  GetCarSteps,
  GetInspectionSteps,
  GetSarlaftSteps,
  SarlaftPersonType,
  StepConfig
} from '../types';
import { shortSarlaftSteps } from './shortSarlaftSteps';
import { inspectionOwnStep } from './inspectionOwnStep';
import { InspectionType } from '../constants/InspectionType';
import { inspectionAutomasStep } from './InspectionAutomasStep';
import { getCommonSarlaftSteps } from './sarlaftSteps';

const STEPS_OF_FIRST_SECTION_OF_CHECKOUT = [StepIds.InspectionOwn, StepIds.Payment];

const getCarSteps: GetCarSteps = (requireCarInformation, optionalSarlaftFields) => {
  const NAME = 'name';
  const carSteps = getCarStepsConfig(optionalSarlaftFields);
  const carStepsToShow = requireCarInformation.map(fieldName => {
    const carField = carSteps.map(step =>
      step.fields?.find(field => field[NAME] === `sarlaft.insured.car.${fieldName}`)
    );
    return carField[0] as IFieldProps | IFieldProps[];
  });
  const { fields, ...values } = carSteps[0];
  const newCarSteps = { fields: carStepsToShow, ...values };
  const carSection = [newCarSteps];
  return carSection || [];
};

const getInspectionSteps: GetInspectionSteps = ({
  is0Km,
  renewalFlow = false,
  product,
  createInspection,
  showInspectionCenter,
  isTLC
}) => {
  const inspectionType = getInspectionType(product);

  if (!is0Km && !renewalFlow && !isTLC && !inspectionType.includes(InspectionType.Own)) {
    return [
      inspectionAutomasStep(
        createInspection as any,
        showInspectionCenter,
        inspectionType as AllowedInspectionType
      )
    ];
  }

  return [];
};

const getSarlaftSteps: GetSarlaftSteps = ({
  carValuation = '0',
  nextStep,
  sarlaftPersonType,
  sarlaftType,
  optionalSarlaftFields
}) => {
  if (sarlaftType === CicoSarlaftType.SHORT) {
    return shortSarlaftSteps(sarlaftPersonType, nextStep, optionalSarlaftFields);
  }

  return [
    ...insuredSteps(sarlaftPersonType, optionalSarlaftFields),
    ...occupationSteps({ sarlaftPersonType, optionalFieldNames: optionalSarlaftFields }),
    ...financialSteps(sarlaftPersonType, carValuation, optionalSarlaftFields),
    ...publicResourcesSteps(sarlaftPersonType, optionalSarlaftFields),
    ...foreignTaxation(sarlaftPersonType, optionalSarlaftFields),
    ...insuranceSteps(sarlaftPersonType, nextStep, optionalSarlaftFields)
  ];
};

export const stepsConfig: StepConfig = ({
  additionalDocumentFilename,
  bffURL,
  fulfillmentPresenterURL,
  createInspection,
  fileHandler,
  isDigitalSignatureEnabled,
  sarlaftSignatureRequired,
  offer,
  onGetQuestions,
  ownerViewing,
  paymentMethods,
  questions,
  quoteForm,
  requireCarInformation,
  sarlaftType,
  showInspectionCenter,
  showAdditionalDocument,
  initialStep,
  optionalSarlaftFields
}) => {
  const { product, offerId } = offer;
  const { car, id, is0Km, renewal, plate } = quoteForm;

  const insuredNextStep = requireCarInformation?.length !== 0 ? StepIds.Car : StepIds.Files;
  const valuation = car ? String(car.valuation) : '';
  const isTLC = isTLCPlate(plate);

  let stepPreviousToFileStep = StepIds.InsuredInsurance;
  if (sarlaftType === CicoSarlaftType.SHORT) stepPreviousToFileStep = StepIds.InsuredAddress;
  if (requireCarInformation?.length !== 0) stepPreviousToFileStep = StepIds.Car;

  if (!initialStep || STEPS_OF_FIRST_SECTION_OF_CHECKOUT.includes(initialStep)) {
    return [
      inspectionOwnStep,
      ...paymentStep({
        bffURL,
        fileHandler,
        offer,
        ownerViewing,
        paymentMethods,
        funnelId: id
      })
    ];
  }

  return [
    ...getInspectionSteps({
      createInspection,
      isTLC,
      product,
      showInspectionCenter,
      is0Km: is0Km as boolean,
      renewalFlow: renewal as boolean
    }),
    ...getCommonSarlaftSteps(optionalSarlaftFields),
    ...getSarlaftSteps({
      sarlaftType,
      carValuation: valuation,
      nextStep: insuredNextStep,
      sarlaftPersonType: SarlaftPersonType.Insured,
      optionalSarlaftFields
    }),
    ...getSarlaftSteps({
      sarlaftType,
      nextStep: StepIds.Files,
      sarlaftPersonType: SarlaftPersonType.Payer,
      optionalSarlaftFields
    }),
    ...getCarSteps(requireCarInformation, optionalSarlaftFields),
    ...getDigitalSignatureSteps({
      onGetQuestions,
      questions,
      sarlaftSignatureRequired,
      BFFURL: bffURL,
      additionalDocumentName: additionalDocumentFilename
    }),
    ...getFileSteps({
      additionalDocumentFilename,
      bffURL,
      fulfillmentPresenterURL,
      fileHandler,
      isDigitalSignatureEnabled,
      sarlaftSignatureRequired,
      offerId,
      onGetQuestions,
      quoteId: id,
      showAdditionalDocument,
      previousStep: stepPreviousToFileStep,
      quoteForm
    })
  ];
};
