import luhn from 'fast-luhn';
import { Validator } from '@comparaonline/ui-forms-validators/dist/types/Validator';
import { errorCode } from '@comparaonline/ui-forms-validators/dist/validators/commons/creditCard';
import { cleanCardNumber, getCardType } from '../helpers/creditCard';
import { CARD_EXPEDITION_FORMAT } from '../constants';
import { Checkout } from '@comparaonline/cico-funnel-schema';

export const creditCardCode = errorCode;

const cardNumberIsEncrypted = cardNumber => cardNumber.includes('*');

export const creditCard: Validator<string, Checkout> = (value, checkoutValues) => {
  if (cardNumberIsEncrypted(value) && !!checkoutValues.payment.creditCard!.token) return;
  const cardType = getCardType(value);
  const cleanedCardNumber = cleanCardNumber(value);
  const isLuhnValid = cardType && cardType.luhn ? luhn(cleanedCardNumber) : true;
  const isValid = cardType && isLuhnValid && cardType.test(cleanedCardNumber, false);
  return isValid ? undefined : { errorCode: creditCardCode };
};

const MAX_YEAR_IN_FUTURE = 10;

export const creditCardExpeditionCode = 'CREDIT_CARD_EXPEDITION_ERROR';
export const creditCardExpedition: Validator<string> = value => {
  const month = Number(value.slice(0, 2));
  const year = Number(value.slice(2, 4));

  const today = new Date();
  const currentMonth = today.getMonth() + 1;
  const shortYear = today
    .getFullYear()
    .toString()
    .substr(-2);
  const currentYear = Number(shortYear);
  const maxYear = currentYear + MAX_YEAR_IN_FUTURE;

  const isCurrentYear = year === currentYear;
  const isFutureYear = year > currentYear && year <= maxYear;
  const isYearValid = isCurrentYear || isFutureYear;

  const isNotAPastMonth = isCurrentYear ? month >= currentMonth : true;
  const isMonthValid = isNotAPastMonth && month >= 1 && month < 13;

  return isYearValid && isMonthValid ? undefined : { errorCode: creditCardExpeditionCode };
};

export const invalidCardExpeditionFormatCode = `invalidCardExpeditionFormat`;
export const invalidCardExpeditionFormat: Validator<string> = value => {
  return String(value).match(CARD_EXPEDITION_FORMAT)
    ? undefined
    : { errorCode: invalidCardExpeditionFormatCode };
};

const ALLOWED_CREDIT_CARD_TYPES = ['American Express', 'Diners Club', 'Mastercard', 'Visa'];
export const creditCardTypeCode = 'CREDIT_CARD_TYPE_ERROR';
export const creditCardType: Validator<string, Checkout> = (value, checkoutValues) => {
  if (!value || (cardNumberIsEncrypted(value) && !!checkoutValues.payment.creditCard!.token)) {
    return;
  }
  const cardType = getCardType(value);
  return (cardType && ALLOWED_CREDIT_CARD_TYPES.includes(cardType.name)) || isCodensa(value)
    ? undefined
    : { errorCode: creditCardTypeCode };
};

const CODENSAR_BIN = '590712';
function isCodensa(cardNumber: string): boolean {
  return cardNumber.replace(/\D/g, '').startsWith(CODENSAR_BIN);
}
